<?php

namespace Service;

class Order {
    
    protected $connection;
    protected $util;
    protected $precoAPI;
    protected $impressaoPedidoApi;
    protected $caixaAPI;
    protected $localUtil;
	protected $impressaoUtilAPI;
    protected $vendaAPI;
    
    const DELIVERY_ORDER = 'DLV';
    const TOGO_ORDER = 'TGO';
    const RESTAURANT_ORDER = 'MES';
    const COMANDA_ORDER = 'CMD';
	const ITEM_CANCELADO = '6';
    
	public function __construct($connection, 
                                \Odhen\API\Util\Util $util, 
                                \Odhen\API\Service\Preco $precoAPI,
								\Odhen\API\Service\ImpressaoPedido $impressaoPedidoApi,
								\Odhen\API\Service\Caixa $caixaAPI, 
                                \Util\Util $localUtil,
								\Odhen\API\Lib\ImpressaoUtil $impressaoUtilAPI,
								\Odhen\API\Service\Venda $vendaAPI){
        $this->connection = $connection;
        $this->util = $util;
        $this->precoAPI = $precoAPI;
        $this->impressaoPedidoApi = $impressaoPedidoApi;
        $this->caixaAPI = $caixaAPI;
        $this->localUtil = $localUtil;
		$this->impressaoUtilAPI = $impressaoUtilAPI;
		$this->vendaAPI = $vendaAPI;
    }
    
    public function getStandardSeller($CDFILIAL) {
		$params = array(
			'CDFILIAL' => $CDFILIAL
		);
		try {
			if ($this->util->databaseIsOracle()) $standardSeller = $this->connection->fetchAssoc(\Util\Query::ORACLE_STANDARD_SELLER, $params);
			else $standardSeller = $this->connection->fetchAssoc(\Util\Query::SQL_STANDARD_SELLER, $params);
			if(!$standardSeller) {
				$return = array(
					'error' => true,
					'message' => 'Parametrizacao de vendedor padrao incompleta ou inexistente'
				);
            } elseif (empty($standardSeller['CDCAIXA'])) {
                $return = array(
					'error' => true,
					'message' => 'Nao ha caixa relacionado ao vendedor padrao. Verifique o Cad. Vendedor(Garçom)'
				);
            } elseif (empty($standardSeller['CDOPERADOR'])) {
                $return = array(
					'error' => true,
					'message' => 'Nao foi encontrado operador relacionado ao vendedor padrao. Verifique o Cad. Vendedor(Garçom)'
				);
            } elseif (empty($standardSeller['NRMESA'])) {
                $return = array(
					'error' => true,
					'message' => 'Mesa padrao nao parametrizada. Verifique a parametrizacao de loja'
				);
			} else {
				$return = array(
					'error' => false,
					'data' => $standardSeller
				);
			}	
		} catch (\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro ao buscar parametros de vendedor padrao. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

    public function insertVendaRest($params){
		try {
            // verifica qual banco de dados está sendo utilizado
            if ($this->util->databaseIsOracle()) $success = $this->connection->executeQuery(\Util\Query::ORACLE_INSERT_VENDAREST, $params);
            else $success = $this->connection->executeQuery(\Util\Query::SQL_INSERT_VENDAREST, $params);
            $return = array('error' => false);
		} catch (\Exception $e) {
			$return = array(
				'error'   => true,
				'message' => 'Erro ao inserir vendarest. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

    public function insertComandaVen($params){
		try {
            if ($this->util->databaseIsOracle()) $success = $this->connection->executeQuery(\Util\Query::ORACLE_INSERT_COMANDAVEN, $params);
            else $success = $this->connection->executeQuery(\Util\Query::SQL_INSERT_COMANDAVEN, $params);
            $return = array('error' => false);
		} catch (\Exception $e) {
			$return = array(
				'error'   => true,
				'message' => 'Erro ao inserir comandaven. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

    public function insertComandaVenEx($params){
		try {
            if ($this->util->databaseIsOracle()) $success = $this->connection->executeQuery(\Util\Query::ORACLE_INSERT_COMANDAVEN_EX, $params);
            else $success = $this->connection->executeQuery(\Util\Query::SQL_INSERT_COMANDAVEN_EX, $params);
            $return = array('error' => false);
		} catch (\Exception $e) {
			$return = array(
				'error'   => true,
				'message' => 'Erro ao inserir comandaven. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

    public function validadeSeqProdcomven($subsidiaryCode, $storeCode, $vendarestSeq, $comandavenSeq){
		try {

            $params = array(
                ':CDFILIAL'    => $subsidiaryCode,
                ':CDLOJA'      => $storeCode,
                ':NRVENDAREST' => $vendarestSeq,
                ':NRCOMANDA'   => $comandavenSeq 
            );

            $data = $this->connection->fetchAssoc(\Util\Query::CHECK_NRPRODCOMVEN, $params);

            $nrprodcomvenSeq = !empty($data['NRPRODCOMVEN']) ? $data['NRPRODCOMVEN'] : 0;

            $return = array(
                'error' => false,
                'data' => $nrprodcomvenSeq
            );

		} catch (\Exception $e) {
			$return = array(
				'error'   => true,
				'message' => 'Erro ao consultar sequencial da comanda. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

    public function formatItems($items, $orderType, $subsidiaryCode, $vendarestSeq, $comandavenSeq, $storeCode, $idcontroprod, $sellerPos, $sellerCode, $orderCPF, $consumerName, $pagerId, $holdTime) {

        $finalItemsArray = [
            "ITCOMANDAVEN" => [],
            "ITCOMANDAEST" => []
        ];

        $itemSeq = $this->generateItcomandavenSeq($subsidiaryCode, $comandavenSeq);
        $comboSeq = $itemSeq;

        //$isNotFirstSeq = $this->validadeSeqProdcomven($subsidiaryCode, $storeCode, $vendarestSeq, $comandavenSeq);
        //if($isNotFirstSeq > 0){
        //    $itemSeq = $isNotFirstSeq['data'] + 1;
        //    $comboSeq = (int)$isNotFirstSeq['data'];   
        //}
        
        $orderItemsSeq = $this->generateItemsSeq($subsidiaryCode);

        foreach($items as $item) {
            //força quantidade 1 no combo para forçar o envio do combo aberto
            if(!empty($item['childItems'])) {
                $item['quantity'] = 1;
            }

            $itemArray = $this->formatItem($item, $orderType, $subsidiaryCode, $vendarestSeq, $comandavenSeq, $storeCode, $idcontroprod, $sellerPos, $sellerCode, $orderCPF, $consumerName, $pagerId, $holdTime);

            $obsToInsert = [];
            foreach($itemArray['pricedObs'] as $pricedObs) {
                array_push($obsToInsert, $pricedObs);
            }
            unset($itemArray['pricedObs']);

            if(empty($item['childItems'])) {
                $itemArray['NRPRODCOMVEN'] = str_pad($itemSeq, 6, '0', STR_PAD_LEFT);
                $itemArray['NRSEQPRODCOM'] = null;
                $itemArray['NRSEQPRODCUP'] = $orderItemsSeq;
                $itemArray['CDPRODPROMOCAO'] = null;
                $itemArray['IDIMPPRGRPROMO'] = 'N';
                $itemSeq = $this->generateItcomandavenSeq($subsidiaryCode, $comandavenSeq);
                
                foreach($itemArray['observations'] as &$itemObs){
                    $itemObs['NRPRODCOMVEN'] = $itemArray['NRPRODCOMVEN'];
                }

                array_push($finalItemsArray['ITCOMANDAVEN'], $itemArray);
            }

            else if($item['productPrint'] === 'S'){
            $itemArray['NRPRODCOMVEN'] = str_pad($itemSeq, 6, '0', STR_PAD_LEFT);
            $itemArray['NRSEQPRODCOM'] = $this->localUtil->formatNrSeqProdCom($itemArray['NRPRODCOMVEN']);
            $itemArray['NRSEQPRODCUP'] = $orderItemsSeq;
            $itemArray['CDPRODPROMOCAO'] = $item['productCode'];
            $itemArray['IDIMPPRGRPROMO'] = 'N';
            $itemSeq = $this->generateItcomandavenSeq($subsidiaryCode, $comandavenSeq);
            
            foreach($itemArray['observations'] as &$itemObs){
                $itemObs['NRPRODCOMVEN'] = $itemArray['NRPRODCOMVEN'];
            }

            array_push($finalItemsArray['ITCOMANDAVEN'], $itemArray);
            }

            foreach($obsToInsert as $obsToInsertIt) {
                $obsToInsertIt['NRPRODCOMVEN'] = str_pad($itemSeq, 6, '0', STR_PAD_LEFT);
                $obsToInsertIt['NRSEQPRODCOM'] = null;
                $obsToInsertIt['NRSEQPRODCUP'] = $orderItemsSeq;
                $obsToInsertIt['CDPRODPROMOCAO'] = null;
                $obsToInsertIt['IDIMPPRGRPROMO'] = 'N'; //verificar
                $obsToInsertIt['IDOBSERVACAOAD'] = 'S';
                $itemSeq = $this->generateItcomandavenSeq($subsidiaryCode, $comandavenSeq);
                array_push($finalItemsArray['ITCOMANDAVEN'], $obsToInsertIt);
            }

            if(!empty($item['childItems'])) {
                // $comboSeq = 1;
                //$comboSeq++;

                $grouppedChildItems = [];

                foreach($item['childItems'] as $childItem) {
                    if(!empty($grouppedChildItems[$childItem['productCode']])) {
                        $grouppedChildItems[$childItem['productCode']]['quantity'] += $childItem['quantity'];
                    } else{
                        $grouppedChildItems[$childItem['productCode']] = $childItem;
                    }
                }

                $nrSeqProdComEx = null;
                foreach($grouppedChildItems as $childItem) {
                    $childItemArray = $this->formatItem($childItem, $orderType, $subsidiaryCode, $vendarestSeq, $comandavenSeq, $storeCode, $idcontroprod, $sellerPos, $sellerCode, $orderCPF, $consumerName, $pagerId, $holdTime);

                    $childObsToInsert = [];
                    foreach($childItemArray['pricedObs'] as $pricedObs) {
                        array_push($childObsToInsert, $pricedObs);
                    }
                    unset($childItemArray['pricedObs']);

                    if(empty($childItem['childItems'])) {
                        if($item['productPrint'] === 'N') {
                            $childItemArray['NRPRODCOMVEN'] = str_pad($itemSeq, 6, '0', STR_PAD_LEFT);

                            if(empty($nrSeqProdComEx)) {
                                $nrSeqProdComEx = $this->localUtil->formatNrSeqProdCom($childItemArray['NRPRODCOMVEN']);
                            }

                            $childItemArray['NRSEQPRODCOM'] = $nrSeqProdComEx; //$this->localUtil->formatNrSeqProdCom($childItemArray['NRPRODCOMVEN']);
                            // $childItemArray['NRSEQPRODCOM'] = '001';
                            $childItemArray['NRSEQPRODCUP'] = $orderItemsSeq;
                            $childItemArray['CDPRODPROMOCAO'] = $item['productCode'];
                            // $childItemArray['IDIMPPRGRPROMO'] = $childItem['productPrint'];
                            $childItemArray['IDIMPPRGRPROMO'] = $item['productPrint'];
                            $itemSeq = $this->generateItcomandavenSeq($subsidiaryCode, $comandavenSeq);
                            $comboSeq++;

                            foreach($childItemArray['observations'] as &$childItemObs){
                                $childItemObs['NRPRODCOMVEN'] = $childItemArray['NRPRODCOMVEN'];
                            }
                            
                            array_push($finalItemsArray['ITCOMANDAVEN'], $childItemArray);
                        }
                        
                        if($item['productPrint'] === 'S') {
                            if ($childItem['productPrint'] === 'S' && $childItem['groupPrint'] === 'S'){
                                $childItemArray['NRPRODCOMVEN'] = str_pad($itemSeq, 6, '0', STR_PAD_LEFT);
                                $childItemArray['NRSEQPRODCOM'] = null;
                                $childItemArray['NRSEQPRODCUP'] = $orderItemsSeq;
                                $childItemArray['CDPRODPROMOCAO'] = null;
                                $childItemArray['IDIMPPRGRPROMO'] = $childItem['productPrint'];

                                foreach($childItemArray['observations'] as &$childItemObs){
                                $childItemObs['NRPRODCOMVEN'] = $childItemArray['NRPRODCOMVEN'];
                                }
                                
                                array_push($finalItemsArray['ITCOMANDAVEN'], $childItemArray);
                                $itemSeq = $this->generateItcomandavenSeq($subsidiaryCode, $comandavenSeq);
                            }
                            
                            $childItemArray['NRPRODCOMVEN'] = $itemArray['NRPRODCOMVEN'];
                            $childItemArray['QTPROCOMEST'] = $childItemArray['QTPRODCOMVEN'];
                            $childItemArray['VRPRECCOMEST'] = $childItemArray['VRPRECCOMVEN'];
                            $childItemArray['VRDESITCOMEST'] = $childItemArray['VRDESCCOMVEN'];
                            $childItemArray['IDIMPPRGRPROMO'] = $childItem['productPrint'];
                            $childItemArray['TXPRODCOMVENEST'] = $childItemArray['TXPRODCOMVEN'];
                            $childItemArray['VRACRITCOMEST'] = 0.00;
                            unset($childItemArray['QTPRODCOMVEN']);
                            unset($childItemArray['VRPRECCOMVEN']);
                            unset($childItemArray['TXPRODCOMVEN']);
                            unset($childItemArray['IDSTPRCOMVEN']);
                            unset($childItemArray['VRDESCCOMVEN']);
                            unset($childItemArray['NRLUGARMESA']);
                            unset($childItemArray['IDPRODIMPFIS']);
                            unset($childItemArray['CDLOJA']);
                            unset($childItemArray['VRACRCOMVEN']);
                            unset($childItemArray['VRPRECCLCOMVEN']);
                            unset($childItemArray['CDCAIXACOLETOR']);
                            unset($childItemArray['CDPRODPROMOCAO']);
                            unset($childItemArray['CDVENDEDOR']);
                            unset($childItemArray['NRINSCRCONS']);
                            unset($childItemArray['CDSENHAPED']);
                            unset($childItemArray['NRATRAPRODCOVE']);
                            unset($childItemArray['IDORIGEMVENDA']);
                            unset($childItemArray['DSOBSPEDDIGCMD']);

                            foreach($childItemArray['observations'] as &$childItemObs){
                                $childItemObs['NRPRODCOMVEN'] = $childItemArray['NRPRODCOMVEN'];
                                $childItemObs['CDPRODUTO'] = $childItemArray['CDPRODUTO'];
                            }

                            array_push($finalItemsArray['ITCOMANDAEST'], $childItemArray);
                        }
                    }

                    foreach($childObsToInsert as $childObsToInsertIt) {
                        $childObsToInsertIt['NRPRODCOMVEN'] = str_pad($itemSeq, 6, '0', STR_PAD_LEFT);
                        $childObsToInsertIt['NRSEQPRODCOM'] = null;
                        $childObsToInsertIt['NRSEQPRODCUP'] = $orderItemsSeq;
                        $childObsToInsertIt['CDPRODPROMOCAO'] = null;
                        $childObsToInsertIt['IDIMPPRGRPROMO'] = 'N'; //verificar
                        $childObsToInsertIt['IDOBSERVACAOAD'] = 'S';
                        $itemSeq = $this->generateItcomandavenSeq($subsidiaryCode, $comandavenSeq);
                        array_push($finalItemsArray['ITCOMANDAVEN'], $childObsToInsertIt);
                    }

                    if(!empty($childItem['childItems'])) {
                        foreach($childItem['childItems'] as $childCombo) {
                            $childCombo['quantity'] = $childItem['quantity'] * $childCombo['quantity'];
                            $childComboArray = $this->formatItem($childCombo, $orderType, $subsidiaryCode, $vendarestSeq, $comandavenSeq, $storeCode, $idcontroprod, $sellerPos, $sellerCode, $orderCPF, $consumerName, $pagerId, $holdTime);

                            $childComboObsToInsert = [];
                            foreach($childComboArray['pricedObs'] as $pricedObs) {
                                array_push($childComboObsToInsert, $pricedObs);
                            }
                            unset($childComboArray['pricedObs']);

                            if($item['productPrint'] === 'N') {
                                $childComboArray['NRPRODCOMVEN'] = str_pad($itemSeq, 6, '0', STR_PAD_LEFT);
                                $childComboArray['NRSEQPRODCOM'] = $this->localUtil->formatNrSeqProdCom($childComboArray['NRPRODCOMVEN']);
                                $childComboArray['NRSEQPRODCUP'] = $orderItemsSeq;
                                $childComboArray['CDPRODPROMOCAO'] = $item['productCode'];
                                $childComboArray['IDIMPPRGRPROMO'] = $childCombo['productPrint'];
                                $itemSeq = $this->generateItcomandavenSeq($subsidiaryCode, $comandavenSeq);
                                $comboSeq = $itemSeq;

                                foreach($childComboArray['observations'] as &$childComboObs){
                                    $childComboObs['NRPRODCOMVEN'] = $childComboArray['NRPRODCOMVEN'];
                                }
                                die;
                                array_push($finalItemsArray['ITCOMANDAVEN'], $childComboArray);
                            }
                            
                            if($item['productPrint'] === 'S') {

                            if(!isset($childItem['groupPrint'])){
                                throw new \Exception('Nao foi possivel salvar o pedido.');
                            }

                            if($childItem['groupPrint'] === 'S'){
                                if(($childCombo['productCode'] === $childItem['productCode']) && ($childCombo['productCode'] != $item['productCode'])) {
                                //imprime sim
                                //insere comandaven
                                $childComboArray['NRPRODCOMVEN'] = str_pad($itemSeq, 6, '0', STR_PAD_LEFT);
                                $childComboArray['NRSEQPRODCOM'] = null;
                                $childComboArray['NRSEQPRODCUP'] = $orderItemsSeq;
                                $childComboArray['CDPRODPROMOCAO'] = null;
                                $childComboArray['IDIMPPRGRPROMO'] = 'S';
                                $itemSeq = $this->generateItcomandavenSeq($subsidiaryCode, $comandavenSeq);
            
                                foreach($childComboArray['observations'] as &$childComboObs){
                                    $childComboObs['NRPRODCOMVEN'] = $childComboArray['NRPRODCOMVEN'];
                                }
            
                                array_push($finalItemsArray['ITCOMANDAVEN'], $childComboArray);

                                //insere comandaest
                                $childComboArray['NRPRODCOMVEN'] = $itemArray['NRPRODCOMVEN'];
                                    $childComboArray['QTPROCOMEST'] = $childComboArray['QTPRODCOMVEN'];
                                    $childComboArray['VRPRECCOMEST'] = $childComboArray['VRPRECCOMVEN'];
                                    $childComboArray['VRDESITCOMEST'] = $childComboArray['VRDESCCOMVEN'];
                                    $childComboArray['IDIMPPRGRPROMO'] = 'S';
                                    $childComboArray['TXPRODCOMVENEST'] = $childComboArray['TXPRODCOMVEN'];
                                    $childComboArray['VRACRITCOMEST'] = 0.00;
                                    unset($childComboArray['QTPRODCOMVEN']);
                                    unset($childComboArray['VRPRECCOMVEN']);
                                    unset($childComboArray['TXPRODCOMVEN']);
                                    unset($childComboArray['IDSTPRCOMVEN']);
                                    unset($childComboArray['VRDESCCOMVEN']);
                                    unset($childComboArray['NRLUGARMESA']);
                                    unset($childComboArray['IDPRODIMPFIS']);
                                    unset($childComboArray['CDLOJA']);
                                    unset($childComboArray['VRACRCOMVEN']);
                                    unset($childComboArray['VRPRECCLCOMVEN']);
                                    unset($childComboArray['CDCAIXACOLETOR']);
                                    unset($childComboArray['CDPRODPROMOCAO']);
                                    unset($childComboArray['CDVENDEDOR']);
                                    unset($childComboArray['NRINSCRCONS']);
                                    unset($childComboArray['CDSENHAPED']);
                                    unset($childComboArray['NRATRAPRODCOVE']);
                                    unset($childComboArray['IDORIGEMVENDA']);
                                    unset($childComboArray['DSOBSPEDDIGCMD']);
            
                                    foreach($childComboArray['observations'] as &$childComboObs){
                                    $childComboObs['NRPRODCOMVEN'] = $childComboArray['NRPRODCOMVEN'];
                                    $childComboObs['CDPRODUTO'] = $childComboArray['CDPRODUTO'];
                                    }
            
                                    array_push($finalItemsArray['ITCOMANDAEST'], $childComboArray);
                                } else{
                                //olha o imprime de $childCombo['productPrint']
                                if($childCombo['productPrint'] === 'S'){
                                    //insere comandaven
                                    $childComboArray['NRPRODCOMVEN'] = str_pad($itemSeq, 6, '0', STR_PAD_LEFT);
                                    $childComboArray['NRSEQPRODCOM'] = null;
                                    $childComboArray['NRSEQPRODCUP'] = $orderItemsSeq;
                                    $childComboArray['CDPRODPROMOCAO'] = null;
                                    $childComboArray['IDIMPPRGRPROMO'] = $childCombo['productPrint'];
                                    $itemSeq = $this->generateItcomandavenSeq($subsidiaryCode, $comandavenSeq);
            
                                    foreach($childComboArray['observations'] as &$childComboObs){
                                    $childComboObs['NRPRODCOMVEN'] = $childComboArray['NRPRODCOMVEN'];
                                    }
            
                                    array_push($finalItemsArray['ITCOMANDAVEN'], $childComboArray);

                                    //insere comandaest
                                    $childComboArray['NRPRODCOMVEN'] = $itemArray['NRPRODCOMVEN'];
                                    $childComboArray['QTPROCOMEST'] = $childComboArray['QTPRODCOMVEN'];
                                    $childComboArray['VRPRECCOMEST'] = $childComboArray['VRPRECCOMVEN'];
                                    $childComboArray['VRDESITCOMEST'] = $childComboArray['VRDESCCOMVEN'];
                                    $childComboArray['IDIMPPRGRPROMO'] = $childCombo['productPrint'];
                                    $childComboArray['TXPRODCOMVENEST'] = $childComboArray['TXPRODCOMVEN'];
                                    $childComboArray['VRACRITCOMEST'] = 0.00;
                                    unset($childComboArray['QTPRODCOMVEN']);
                                    unset($childComboArray['VRPRECCOMVEN']);
                                    unset($childComboArray['TXPRODCOMVEN']);
                                    unset($childComboArray['IDSTPRCOMVEN']);
                                    unset($childComboArray['VRDESCCOMVEN']);
                                    unset($childComboArray['NRLUGARMESA']);
                                    unset($childComboArray['IDPRODIMPFIS']);
                                    unset($childComboArray['CDLOJA']);
                                    unset($childComboArray['VRACRCOMVEN']);
                                    unset($childComboArray['VRPRECCLCOMVEN']);
                                    unset($childComboArray['CDCAIXACOLETOR']);
                                    unset($childComboArray['CDPRODPROMOCAO']);
                                    unset($childComboArray['CDVENDEDOR']);
                                    unset($childComboArray['NRINSCRCONS']);
                                    unset($childComboArray['CDSENHAPED']);
                                    unset($childComboArray['NRATRAPRODCOVE']);
                                    unset($childComboArray['IDORIGEMVENDA']);
                                    unset($childComboArray['DSOBSPEDDIGCMD']);
            
                                    foreach($childComboArray['observations'] as &$childComboObs){
                                    $childComboObs['NRPRODCOMVEN'] = $childComboArray['NRPRODCOMVEN'];
                                    $childComboObs['CDPRODUTO'] = $childComboArray['CDPRODUTO'];
                                    }
            
                                    array_push($finalItemsArray['ITCOMANDAEST'], $childComboArray);
                                } else{
                                    //insere comandaest
                                    $childComboArray['NRPRODCOMVEN'] = $itemArray['NRPRODCOMVEN'];
                                    $childComboArray['QTPROCOMEST'] = $childComboArray['QTPRODCOMVEN'];
                                    $childComboArray['VRPRECCOMEST'] = $childComboArray['VRPRECCOMVEN'];
                                    $childComboArray['VRDESITCOMEST'] = $childComboArray['VRDESCCOMVEN'];
                                    $childComboArray['IDIMPPRGRPROMO'] = $childCombo['productPrint'];
                                    $childComboArray['TXPRODCOMVENEST'] = $childComboArray['TXPRODCOMVEN'];
                                    $childComboArray['VRACRITCOMEST'] = 0.00;
                                    unset($childComboArray['QTPRODCOMVEN']);
                                    unset($childComboArray['VRPRECCOMVEN']);
                                    unset($childComboArray['TXPRODCOMVEN']);
                                    unset($childComboArray['IDSTPRCOMVEN']);
                                    unset($childComboArray['VRDESCCOMVEN']);
                                    unset($childComboArray['NRLUGARMESA']);
                                    unset($childComboArray['IDPRODIMPFIS']);
                                    unset($childComboArray['CDLOJA']);
                                    unset($childComboArray['VRACRCOMVEN']);
                                    unset($childComboArray['VRPRECCLCOMVEN']);
                                    unset($childComboArray['CDCAIXACOLETOR']);
                                    unset($childComboArray['CDPRODPROMOCAO']);
                                    unset($childComboArray['CDVENDEDOR']);
                                    unset($childComboArray['NRINSCRCONS']);
                                    unset($childComboArray['CDSENHAPED']);
                                    unset($childComboArray['NRATRAPRODCOVE']);
                                    unset($childComboArray['IDORIGEMVENDA']);
                                    unset($childComboArray['DSOBSPEDDIGCMD']);
            
                                    foreach($childComboArray['observations'] as &$childComboObs){
                                    $childComboObs['NRPRODCOMVEN'] = $childComboArray['NRPRODCOMVEN'];
                                    $childComboObs['CDPRODUTO'] = $childComboArray['CDPRODUTO'];
                                    }
            
                                    array_push($finalItemsArray['ITCOMANDAEST'], $childComboArray);
                                }
                                }
                            } else{
                                if($childCombo['productPrint'] === 'S'){
                                //olha o imprime de $childCombo['productPrint']
                                //insere comandaven
                                $childComboArray['NRPRODCOMVEN'] = str_pad($itemSeq, 6, '0', STR_PAD_LEFT);
                                $childComboArray['NRSEQPRODCOM'] = null;
                                $childComboArray['NRSEQPRODCUP'] = $orderItemsSeq;
                                $childComboArray['CDPRODPROMOCAO'] = null;
                                $childComboArray['IDIMPPRGRPROMO'] = $childCombo['productPrint'];
                                $itemSeq = $this->generateItcomandavenSeq($subsidiaryCode, $comandavenSeq);
            
                                foreach($childComboArray['observations'] as &$childComboObs){
                                    $childComboObs['NRPRODCOMVEN'] = $childComboArray['NRPRODCOMVEN'];
                                }
            
                                array_push($finalItemsArray['ITCOMANDAVEN'], $childComboArray);

                                //insere comandaest
                                $childComboArray['NRPRODCOMVEN'] = $itemArray['NRPRODCOMVEN'];
                                $childComboArray['QTPROCOMEST'] = $childComboArray['QTPRODCOMVEN'];
                                $childComboArray['VRPRECCOMEST'] = $childComboArray['VRPRECCOMVEN'];
                                $childComboArray['VRDESITCOMEST'] = $childComboArray['VRDESCCOMVEN'];
                                $childComboArray['IDIMPPRGRPROMO'] = $childCombo['productPrint'];
                                $childComboArray['TXPRODCOMVENEST'] = $childComboArray['TXPRODCOMVEN'];
                                $childComboArray['VRACRITCOMEST'] = 0.00;
                                unset($childComboArray['QTPRODCOMVEN']);
                                unset($childComboArray['VRPRECCOMVEN']);
                                unset($childComboArray['TXPRODCOMVEN']);
                                unset($childComboArray['IDSTPRCOMVEN']);
                                unset($childComboArray['VRDESCCOMVEN']);
                                unset($childComboArray['NRLUGARMESA']);
                                unset($childComboArray['IDPRODIMPFIS']);
                                unset($childComboArray['CDLOJA']);
                                unset($childComboArray['VRACRCOMVEN']);
                                unset($childComboArray['VRPRECCLCOMVEN']);
                                unset($childComboArray['CDCAIXACOLETOR']);
                                unset($childComboArray['CDPRODPROMOCAO']);
                                unset($childComboArray['CDVENDEDOR']);
                                unset($childComboArray['NRINSCRCONS']);
                                unset($childComboArray['CDSENHAPED']);
                                unset($childComboArray['NRATRAPRODCOVE']);
                                unset($childComboArray['IDORIGEMVENDA']);
                                unset($childComboArray['DSOBSPEDDIGCMD']);
            
                                foreach($childComboArray['observations'] as &$childComboObs){
                                    $childComboObs['NRPRODCOMVEN'] = $childComboArray['NRPRODCOMVEN'];
                                    $childComboObs['CDPRODUTO'] = $childComboArray['CDPRODUTO'];
                                }
            
                                array_push($finalItemsArray['ITCOMANDAEST'], $childComboArray);
                                } else{
                                //olha o imprime de $childItem['groupPrint']
                                if($childItem['groupPrint'] === 'S'){
                                    //insere comandaven
                                    $childComboArray['NRPRODCOMVEN'] = str_pad($itemSeq, 6, '0', STR_PAD_LEFT);
                                    $childComboArray['NRSEQPRODCOM'] = null;
                                    $childComboArray['NRSEQPRODCUP'] = $orderItemsSeq;
                                    $childComboArray['CDPRODPROMOCAO'] = null;
                                    $childComboArray['IDIMPPRGRPROMO'] = $childCombo['productPrint'];
                                    $itemSeq = $this->generateItcomandavenSeq($subsidiaryCode, $comandavenSeq);
            
                                    foreach($childComboArray['observations'] as &$childComboObs){
                                    $childComboObs['NRPRODCOMVEN'] = $childComboArray['NRPRODCOMVEN'];
                                    }
            
                                    array_push($finalItemsArray['ITCOMANDAVEN'], $childComboArray);

                                    //insere comandaest
                                    $childComboArray['NRPRODCOMVEN'] = $itemArray['NRPRODCOMVEN'];
                                    $childComboArray['QTPROCOMEST'] = $childComboArray['QTPRODCOMVEN'];
                                    $childComboArray['VRPRECCOMEST'] = $childComboArray['VRPRECCOMVEN'];
                                    $childComboArray['VRDESITCOMEST'] = $childComboArray['VRDESCCOMVEN'];
                                    $childComboArray['IDIMPPRGRPROMO'] = $childCombo['productPrint'];
                                    $childComboArray['TXPRODCOMVENEST'] = $childComboArray['TXPRODCOMVEN'];
                                    $childComboArray['VRACRITCOMEST'] = 0.00;
                                    unset($childComboArray['QTPRODCOMVEN']);
                                    unset($childComboArray['VRPRECCOMVEN']);
                                    unset($childComboArray['TXPRODCOMVEN']);
                                    unset($childComboArray['IDSTPRCOMVEN']);
                                    unset($childComboArray['VRDESCCOMVEN']);
                                    unset($childComboArray['NRLUGARMESA']);
                                    unset($childComboArray['IDPRODIMPFIS']);
                                    unset($childComboArray['CDLOJA']);
                                    unset($childComboArray['VRACRCOMVEN']);
                                    unset($childComboArray['VRPRECCLCOMVEN']);
                                    unset($childComboArray['CDCAIXACOLETOR']);
                                    unset($childComboArray['CDPRODPROMOCAO']);
                                    unset($childComboArray['CDVENDEDOR']);
                                    unset($childComboArray['NRINSCRCONS']);
                                    unset($childComboArray['CDSENHAPED']);
                                    unset($childComboArray['NRATRAPRODCOVE']);
                                    unset($childComboArray['IDORIGEMVENDA']);
                                    unset($childComboArray['DSOBSPEDDIGCMD']);
            
                                    foreach($childComboArray['observations'] as &$childComboObs){
                                    $childComboObs['NRPRODCOMVEN'] = $childComboArray['NRPRODCOMVEN'];
                                    $childComboObs['CDPRODUTO'] = $childComboArray['CDPRODUTO'];
                                    }
            
                                    array_push($finalItemsArray['ITCOMANDAEST'], $childComboArray);
                                } else{
                                    //insere comandaest
                                    $childComboArray['NRPRODCOMVEN'] = $itemArray['NRPRODCOMVEN'];
                                    $childComboArray['QTPROCOMEST'] = $childComboArray['QTPRODCOMVEN'];
                                    $childComboArray['VRPRECCOMEST'] = $childComboArray['VRPRECCOMVEN'];
                                    $childComboArray['VRDESITCOMEST'] = $childComboArray['VRDESCCOMVEN'];
                                    $childComboArray['IDIMPPRGRPROMO'] = $childCombo['productPrint'];
                                    $childComboArray['TXPRODCOMVENEST'] = $childComboArray['TXPRODCOMVEN'];
                                    $childComboArray['VRACRITCOMEST'] = 0.00;
                                    unset($childComboArray['QTPRODCOMVEN']);
                                    unset($childComboArray['VRPRECCOMVEN']);
                                    unset($childComboArray['TXPRODCOMVEN']);
                                    unset($childComboArray['IDSTPRCOMVEN']);
                                    unset($childComboArray['VRDESCCOMVEN']);
                                    unset($childComboArray['NRLUGARMESA']);
                                    unset($childComboArray['IDPRODIMPFIS']);
                                    unset($childComboArray['CDLOJA']);
                                    unset($childComboArray['VRACRCOMVEN']);
                                    unset($childComboArray['VRPRECCLCOMVEN']);
                                    unset($childComboArray['CDCAIXACOLETOR']);
                                    unset($childComboArray['CDPRODPROMOCAO']);
                                    unset($childComboArray['CDVENDEDOR']);
                                    unset($childComboArray['NRINSCRCONS']);
                                    unset($childComboArray['CDSENHAPED']);
                                    unset($childComboArray['NRATRAPRODCOVE']);
                                    unset($childComboArray['IDORIGEMVENDA']);
                                    unset($childComboArray['DSOBSPEDDIGCMD']);
            
                                    foreach($childComboArray['observations'] as &$childComboObs){
                                    $childComboObs['NRPRODCOMVEN'] = $childComboArray['NRPRODCOMVEN'];
                                    $childComboObs['CDPRODUTO'] = $childComboArray['CDPRODUTO'];
                                    }
            
                                    array_push($finalItemsArray['ITCOMANDAEST'], $childComboArray);
                                }
                                }
                            }
                        }

                            foreach($childComboObsToInsert as $childComboObsToInsertIt) {
                                $childComboObsToInsertIt['NRPRODCOMVEN'] = str_pad($itemSeq, 6, '0', STR_PAD_LEFT);
                                $childComboObsToInsertIt['NRSEQPRODCOM'] = null;
                                $childComboObsToInsertIt['NRSEQPRODCUP'] = $orderItemsSeq;
                                $childComboObsToInsertIt['CDPRODPROMOCAO'] = null;
                                $childComboObsToInsertIt['IDIMPPRGRPROMO'] = 'N';
                                $childComboObsToInsertIt['IDOBSERVACAOAD'] = 'S';
                                $itemSeq = $this->generateItcomandavenSeq($subsidiaryCode, $comandavenSeq);
                                array_push($finalItemsArray['ITCOMANDAVEN'], $childComboObsToInsertIt);
                            }
                        }
                    }
                }
            }
        }

        return $finalItemsArray;
    }

    private function formatItem($item, $orderType, $subsidiaryCode, $vendarestSeq, $comandavenSeq, $storeCode, $idcontroprod, $sellerPos, $sellerCode, $orderCPF, $consumerName, $pagerId, $holdTime) {
        $itemArray = [];
        $itemArray['CDFILIAL'] = $subsidiaryCode;
        $itemArray['NRVENDAREST'] = $vendarestSeq;
        $itemArray['NRCOMANDA'] = $comandavenSeq;
        $itemArray['CDPRODUTO'] = $item['productCode'];
        $itemArray['NMPRODUTO'] = $item['productName'];
        $itemArray['QTPRODCOMVEN'] = $item['quantity'];
        $itemArray['VRPRECCOMVEN'] = $item['unitValue'];
        $itemArray['VRDESCCOMVEN'] = $item['discount'];
        $itemArray['NRLUGARMESA'] = '01';
        $itemArray['IDPRODIMPFIS'] = 'N';
        $itemArray['CDLOJA'] = $storeCode;
        $itemArray['VRACRCOMVEN'] = 0.00;
        $itemArray['IDSTPRCOMVEN'] = $this->getTheThing($subsidiaryCode, $storeCode, $item['productCode'], $idcontroprod);
        if (!empty($item['toTravel']) && substr($orderType, 0, 3) !== 'TGO' && substr($orderType, 0, 3) !== 'DLV' && substr($orderType, 0, 3) !== 'THR'){
            $itemArray['IDORIGEMVENDA'] = 'TGO_' . $orderType;
        } else{
            $itemArray['IDORIGEMVENDA'] = $orderType;
        }
        $itemArray['TXPRODCOMVEN'] = ' ' . strtoupper($this->removeAcentos($item['note'])) . ';';
        $itemArray['VRPRECCLCOMVEN'] = 0;
        $itemArray['CDCAIXACOLETOR'] = $sellerPos;
        $itemArray['CDVENDEDOR'] = $sellerCode;
        $itemArray['NRINSCRCONS'] = $orderCPF;
        $itemArray['NMCONSVEND'] = $consumerName;
        $itemArray['CDSENHAPED'] = $pagerId;
        $itemArray['NRATRAPRODCOVE'] = $holdTime;
        
        $itemAcrArray = [];
        $itemObsArray = [];

        foreach($item['observations'] as $observation){
            $AcrArray = [];

            if($observation['obsType'] === 'A'){
                $AcrArray['CDFILIAL'] = $subsidiaryCode;
                $AcrArray['NRVENDAREST'] = $vendarestSeq;
                $AcrArray['NRCOMANDA'] = $comandavenSeq;
                $AcrArray['CDPRODUTO'] = $observation['obsProductCode'];
                $AcrArray['NMPRODUTO'] = $observation['obsName'];
                $AcrArray['QTPRODCOMVEN'] = $observation['quantity'];
                $AcrArray['VRPRECCOMVEN'] = $observation['price'];
                $AcrArray['VRDESCCOMVEN'] = 0.00;
                $AcrArray['NRLUGARMESA'] = '01';
                $AcrArray['IDPRODIMPFIS'] = 'N';
                $AcrArray['VRACRCOMVEN'] = 0.00;
                $AcrArray['IDSTPRCOMVEN'] = $this->getTheThing($subsidiaryCode, $storeCode, $observation['obsProductCode'], $idcontroprod);
                $AcrArray['CDLOJA'] = $storeCode;
                $AcrArray['IDORIGEMVENDA'] = $itemArray['IDORIGEMVENDA'];
                $AcrArray['TXPRODCOMVEN'] = '';
                $AcrArray['VRPRECCLCOMVEN'] = 0;
                $AcrArray['CDCAIXACOLETOR'] = $sellerPos;
                $AcrArray['CDVENDEDOR'] = $sellerCode;
                $AcrArray['NRINSCRCONS'] = $orderCPF;
                $AcrArray['NMCONSVEND'] = $consumerName;
                $AcrArray['CDSENHAPED'] = $pagerId;
                $AcrArray['NRATRAPRODCOVE'] = $holdTime;
                $AcrArray['DSOBSPEDDIGCMD'] = $AcrArray['TXPRODCOMVEN'];
                $AcrArray['observations'] = [];

                array_push($itemAcrArray, $AcrArray);
            }

            $itemArray['TXPRODCOMVEN'] .= strtoupper($this->removeAcentos($observation['obsName'])) . '; ';

            $obsArray = [];

            $obsArray['CDFILIAL'] = $subsidiaryCode;
            $obsArray['NRVENDAREST'] = $vendarestSeq;
            $obsArray['NRCOMANDA'] = $comandavenSeq;
            $obsArray['CDGRPOCOR'] = $observation['obsGroupCode'];
            $obsArray['CDOCORR'] = $observation['obsCode'];

            array_push($itemObsArray, $obsArray);
        }

        $itemArray['DSOBSPEDDIGCMD'] = $itemArray['TXPRODCOMVEN'];
        $itemArray['observations'] = $itemObsArray;
        $itemArray['pricedObs'] = $itemAcrArray;

        return $itemArray;
    }

    private function removeAcentos($texto) {
        $table = array('á' => 'a', 'à' => 'a', 'ã' => 'a', 'â' => 'a',
            'é' => 'e', 'ê' => 'e', 'í' => 'i', 'ó' => 'o', 'ô' => 'o',
            'õ' => 'o', 'ú' => 'u', 'ü' => 'u', 'ç' => 'c', 'Á' => 'A',
            'À' => 'A', 'Ã' => 'A', 'Â' => 'A', 'É' => 'E', 'Ê' => 'E',
            'Í' => 'I', 'Ó' => 'O', 'Ô' => 'O', 'Õ' => 'O', 'Ú' => 'U',
            'Ü' => 'U', 'Ç' => 'C_');
        return preg_replace("[^a-zA-Z0-9_]", "", strtr($texto, $table));
    }

    private function getTheThing($subsidiaryCode, $storeCode, $productCode, $idcontroprod){
        $params = array(
            ':CDFILIAL' => $subsidiaryCode,
            ':CDLOJA' => $storeCode,
            ':CDPRODUTO' => $productCode
        );
        $result = $this->connection->fetchAll(\Util\Query::GET_THE_THING, $params);
        if(isset($result[0])){
            $theThing = $result[0];
        }else{
            $theThing = null;
        }
        if ($idcontroprod === 'S' && $theThing !== null) $finalThing = '1';
        else if ($idcontroprod === 'N') $finalThing = '4';
        else $finalThing = $idcontroprod;
        return $finalThing;
    }

    public function saveOrderItems($itemsArray) {
        foreach($itemsArray['ITCOMANDAVEN'] as $comVenItem){
            $params = array(
                ':CDFILIAL'       => $comVenItem['CDFILIAL'],
                ':NRVENDAREST'    => $comVenItem['NRVENDAREST'],
                ':NRCOMANDA'      => $comVenItem['NRCOMANDA'],
                ':NRPRODCOMVEN'   => $comVenItem['NRPRODCOMVEN'],
                ':CDPRODUTO'      => $comVenItem['CDPRODUTO'],
                ':QTPRODCOMVEN'   => $comVenItem['QTPRODCOMVEN'],
                ':VRPRECCOMVEN'   => $comVenItem['VRPRECCOMVEN'],
                ':TXPRODCOMVEN'   => $comVenItem['TXPRODCOMVEN'],
                ':IDSTPRCOMVEN'   => $comVenItem['IDSTPRCOMVEN'],
                ':VRDESCCOMVEN'   => $comVenItem['VRDESCCOMVEN'],
                ':NRLUGARMESA'    => $comVenItem['NRLUGARMESA'],
                ':IDPRODIMPFIS'   => $comVenItem['IDPRODIMPFIS'],
                ':CDLOJA'         => $comVenItem['CDLOJA'],
                ':VRACRCOMVEN'    => $comVenItem['VRACRCOMVEN'],
                ':NRSEQPRODCOM'   => $comVenItem['NRSEQPRODCOM'],
                ':NRSEQPRODCUP'   => $comVenItem['NRSEQPRODCUP'],
                ':VRPRECCLCOMVEN' => $comVenItem['VRPRECCLCOMVEN'],
                ':CDCAIXACOLETOR' => $comVenItem['CDCAIXACOLETOR'],
                ':CDPRODPROMOCAO' => $comVenItem['CDPRODPROMOCAO'],
                ':CDVENDEDOR'     => $comVenItem['CDVENDEDOR'],
                ':NRINSCRCONS'    => $comVenItem['NRINSCRCONS'],
                ':NMCONSVEND'     => $comVenItem['NMCONSVEND'],
                ':CDSENHAPED'     => $comVenItem['CDSENHAPED'],
                ':NRATRAPRODCOVE' => $comVenItem['NRATRAPRODCOVE'],
                ':IDORIGEMVENDA'  => $comVenItem['IDORIGEMVENDA'],
                ':DSOBSPEDDIGCMD' => $comVenItem['DSOBSPEDDIGCMD'],
                ':IDIMPPRGRPROMO' => $comVenItem['IDIMPPRGRPROMO'],
                ':IDOBSERVACAOAD' => (!empty($comVenItem['IDOBSERVACAOAD']) ? $comVenItem['IDOBSERVACAOAD'] : 'N')
            );

            $this->connection->executeQuery(\Util\Query::INSERT_ITCOMANDAVEN_EX, $params);

            foreach($comVenItem['observations'] as $comVenObs){
                $params = array(
                    ':CDFILIAL'     => $comVenObs['CDFILIAL'],
                    ':NRVENDAREST'  => $comVenObs['NRVENDAREST'],
                    ':NRCOMANDA'    => $comVenObs['NRCOMANDA'],
                    ':NRPRODCOMVEN' => $comVenObs['NRPRODCOMVEN'],
                    ':CDGRPOCOR'    => $comVenObs['CDGRPOCOR'],
                    ':CDOCORR'      => $comVenObs['CDOCORR']
                );

                $this->connection->executeQuery(\Util\Query::INSERT_OBSITCOMANDAVEN_EX, $params);
            }
        }

        foreach($itemsArray['ITCOMANDAEST'] as $comEstItem){
            $params = array(
                ':CDFILIAL'       => $comEstItem['CDFILIAL'],
                ':NRVENDAREST'    => $comEstItem['NRVENDAREST'],
                ':NRCOMANDA'      => $comEstItem['NRCOMANDA'],
                ':NRPRODCOMVEN'   => $comEstItem['NRPRODCOMVEN'],
                ':CDPRODUTO'      => $comEstItem['CDPRODUTO'],
                ':QTPROCOMEST'    => $comEstItem['QTPROCOMEST'],
                ':VRPRECCOMEST'   => $comEstItem['VRPRECCOMEST'],
                ':VRDESITCOMEST'  => $comEstItem['VRDESITCOMEST'],
                ':IDIMPPRGRPROMO' => $comEstItem['IDIMPPRGRPROMO'],
                ':VRACRITCOMEST'  => $comEstItem['VRACRITCOMEST'],
                ':TXPRODCOMVENEST'  => $comEstItem['TXPRODCOMVENEST']
            );

            $this->connection->executeQuery(\Util\Query::INSERT_ITCOMANDAEST_EX, $params);

            foreach($comEstItem['observations'] as $comEstObs){
                $params = array(
                    ':CDFILIAL'     => $comEstObs['CDFILIAL'],
                    ':NRVENDAREST'  => $comEstObs['NRVENDAREST'],
                    ':NRCOMANDA'    => $comEstObs['NRCOMANDA'],
                    ':NRPRODCOMVEN' => $comEstObs['NRPRODCOMVEN'],
                    ':CDPRODUTO'    => $comEstObs['CDPRODUTO'],
                    ':CDGRPOCOR'    => $comEstObs['CDGRPOCOR'],
                    ':CDOCORR'      => $comEstObs['CDOCORR']
                );

                $this->connection->executeQuery(\Util\Query::INSERT_OBSITCOMANDAEST_EX, $params);
            }
        }
    }

    public function shareDiscount($initialDiscount, &$products) {

        $idxToApplyDiscount   = [];
        $totalDiscount        = $initialDiscount;
        $discountLeft         = $initialDiscount;
        $productsTotalValue   = "0.00";
        $discountGivenShare   = "0.00";
        $maxValueItem         = "0.00";
        $maxValueIdx          = -1;
        
        foreach($products as &$product) {
            $product['VRDESCCOMVEN']  = 0;
            $product['discountShare'] = "0.00";
        }
        
        if(isset($product))
            unset($product);

        foreach($products as $i => $product) {
            if((bccomp($product["VRPRECCOMVEN"], "0.01", 2) == 1)) { 
                $productsTotalValue = bcadd($productsTotalValue, $product["VRPRECCOMVEN"], 2);
                array_push($idxToApplyDiscount, $i);
            }
        }

        foreach($idxToApplyDiscount as $idxWithDiscount) {
            $discountShare = bcdiv((bcmul($products[$idxWithDiscount]["VRPRECCOMVEN"], "100.00", 2)), $productsTotalValue, 2);
            $products[$idxWithDiscount]["discountShare"] = $discountShare;
            $discountGivenShare = bcadd($discountGivenShare, $discountShare, 2);
        }

        foreach($products as $i => $product) {
            if(bccomp($product['discountShare'], $maxValueItem, 2) == 1) {
                $maxValueIdx  = $i;
                $maxValueItem = $product['discountShare'];
            }            
        }

        if($maxValueIdx != -1) {
            $products[$maxValueIdx]["discountShare"] = bcadd($products[$maxValueIdx]["discountShare"], bcsub("100.00", $discountGivenShare, 2), 2);
        }
    
        foreach($idxToApplyDiscount as $idxWithDiscount) {
            $itemDiscount = bcdiv((bcmul($products[$idxWithDiscount]["discountShare"], $totalDiscount, 2)), "100.00", 2);
            $products[$idxWithDiscount]["VRDESCCOMVEN"] = $itemDiscount;

            $discountLeft = bcsub($discountLeft, $itemDiscount, 2);
        }

        if($maxValueIdx != -1) {
            $products[$maxValueIdx]["VRDESCCOMVEN"] = bcadd($products[$maxValueIdx]["VRDESCCOMVEN"], $discountLeft, 2);
            $discountLeft = "0.00";
        }
        
        $leftDiscount = "0.00";
        foreach($products as &$product) {
            if( (bccomp($product['VRDESCCOMVEN'], $product['VRPRECCOMVEN'], 2) == 1) || (bccomp($product['VRDESCCOMVEN'], $product['VRPRECCOMVEN'], 2) == 0) )  {
                $leftDiscount = bcadd($leftDiscount, bcsub($product['VRDESCCOMVEN'], $product['VRPRECCOMVEN'], 2), 2);
                $leftDiscount = bcadd($leftDiscount, "0.01", 2);
                
                $product['VRDESCCOMVEN'] = bcsub($product['VRDESCCOMVEN'], $leftDiscount, 2);
            }
        }
        
        if(isset($product))
            unset($product);

        foreach($products as &$product){
            $maxAddDisc = (bcsub(bcsub($product['VRPRECCOMVEN'], "0.01", 2), $product['VRDESCCOMVEN'], 2));
            if($maxAddDisc >= $leftDiscount) {
                $addDisc = $leftDiscount;
            } else{
                $addDisc = $maxAddDisc;
            }
            
            $product['VRDESCCOMVEN'] = bcadd($product['VRDESCCOMVEN'], $addDisc, 2);
            $leftDiscount = bcsub($leftDiscount, $addDisc, 2);
        }

        if(isset($product))
            unset($product);

        return $products;
    }


    public function insertMovCaixaDlv($params){
		try {
            if ($this->util->databaseIsOracle()) $success = $this->connection->executeQuery(\Util\Query::ORACLE_INSERT_MOVCAIXADLV, $params);
            else $success = $this->connection->executeQuery(\Util\Query::SQL_INSERT_MOVCAIXADLV, $params);
            $return = array('error' => false);
		} catch (\Exception $e) {
			$return = array(
				'error'   => true,
				'message' => 'Erro ao inserir pagamento. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

    public function insertItComandaVen($params){
		try {
            if ($this->util->databaseIsOracle()) $success = $this->connection->executeQuery(\Util\Query::ORACLE_INSERT_ITCOMANDAVEN, $params);
            else $success = $this->connection->executeQuery(\Util\Query::SQL_INSERT_ITCOMANDAVEN, $params);
            $return = array('error' => false);
		} catch (\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro ao inserir produto. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

	public function insertItComandaEst($params){
		try {
            // if ($this->util->databaseIsOracle()) $success = $this->connection->executeQuery(\Util\Query::INSERT_ITCOMANDAEST, $params);
            // else
			$success = $this->connection->executeQuery(\Util\Query::INSERT_ITCOMANDAEST, $params);
            $return = array('error' => false);
		} catch (\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro ao inserir produto. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

    public function insertItHrComanda($params){
		try {
            if (!$this->util->databaseIsOracle()) $success = $this->connection->executeQuery(\Util\Query::INSERT_ITHRCOMANDA, $params);
            $return = array('error' => false);
		} catch (\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro ao inserir produto(hr). Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

    public function insertObsItComandaVen($params){
		try {
            $success = $this->connection->executeQuery(\Util\Query::INSERT_OBSITCOMANDAVEN, $params);
            $return = array('error' => false);
		} catch (\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro ao inserir observacao. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}
    
    public function isValidCheckinId($checkinId) {
		$params = array(
			'IDCHECKIN' => $checkinId
		);

		$return = array('error' => false);		
		
		try {
			$response = $this->connection->fetchAssoc(\Util\Query::CHECK_VALID_CHECKINID, $params);
			if(!$response) {
				$return['error'] = true;
				$return['message']	= 'CheckinId inválido';
			}	
		} catch (\Exception $e) {
			$return['error'] = true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}
		return $return;
	}
    
    public function getResultCheckinMesPos($checkinId) {
		$params = array(
            'IDCHECKIN' => $checkinId
        );

		$return = array('error' => false);		

		try {
			$response = $this->connection->fetchAssoc(\Util\Query::GET_RESULT_CHECKINMESPOS, $params);
			if($response) {
				$return['data'] = $response;
			} else {
				$return['error'] = true;
				$return['message']	= 'CDFILIAL e NRVENDAREST não encontrados';
			}	
		} catch (\Exception $e) {
			$return['error'] = true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}
		return $return;
	}
    
    public function getResultParavendAndVendedor($CDFILIAL) {
		$params = array(
        	'CDFILIAL' => $CDFILIAL
        );

		$return = array('error' => false);		

		try {
			$response = $this->connection->fetchAssoc(\Util\Query::SQL_STANDARD_SELLER, $params);
			if($response) {
				$return['data'] = $response;
			} else {
				$return['error'] = true;
				$return['message']	= 'CDVENDEDOR, CDCAIXA e CDLOJA não encontrados';
			}	
		} catch (\Exception $e) {
			$return['error'] = true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}
		return $return;
	}

    public function getNrcomanda($NRVENDAREST, $CDFILIAL) {

		$params = array(
        	'NRVENDAREST' => $NRVENDAREST,
            'CDFILIAL' => $CDFILIAL
        );

		$return = array('error' => false);		
		
		try {
			$response = $this->connection->fetchAssoc(\Util\Query::GET_NRCOMANDA, $params);
			if($response) {
				$return['data'] = $response;
			} else {
				$return['error'] = true;
				$return['message']	= 'NRCOMANDA não encontrado';
			}	
		} catch (\Exception $e) {
			$return['error'] = true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}
		return $return;
	}

    public function getNrmesa($CDFILIAL, $NRVENDAREST) {
		$params = array(
        	'CDFILIAL' => $CDFILIAL,
            'NRVENDAREST'	=> $NRVENDAREST
        );

		$return = array('error' => false);		
		
		try {
			$response = $this->connection->fetchAssoc(\Util\Query::GET_NRMESA, $params);
			if($response) {
				$return['data'] = $response;
			} else {
				$return['error'] = true;
				$return['data'] = 'NRMESA não encontrado';
			}	
		} catch (\Exception $e) {
			$return['error'] = true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}
		return $return;
    }
    
    public function insertMovcaixamob($NRSEQMOVMOB, $CDFILIAL, $CDCAIXA, $CDVENDEDOR, $NRVENDAREST, $NRCOMANDA, $NRSEQVENDA, $NRMESA, $NRLUGARMESA, $CDTIPORECE, $IDTIPMOV, $VRMOV, $DSBANDEIRA, $IDTPTEF, $IDUTILTXSERV, $CDCLIENTE, $CDCONSUMIDOR) {
		$params = array(
	        'NRSEQMOVMOB' => $NRSEQMOVMOB, 
	        'CDFILIAL' => $CDFILIAL, 
	        'CDCAIXA' => $CDCAIXA, 
	        'CDVENDEDOR' => $CDVENDEDOR, 
	        'NRVENDAREST' => $NRVENDAREST, 
	        'NRCOMANDA' => $NRCOMANDA, 
	        'NRSEQVENDA' => $NRSEQVENDA, 
	        'NRMESA' => $NRMESA,
	        'NRLUGARMESA' => $NRLUGARMESA, 
	        'CDTIPORECE' => $CDTIPORECE, 
	        'IDTIPMOV' => $IDTIPMOV, 
	        'VRMOV' => $VRMOV, 
	        'DSBANDEIRA' => $DSBANDEIRA, 
	        'IDTPTEF' => $IDTPTEF,
	        'IDUTILTXSERV' => $IDUTILTXSERV,
	        'CDCLIENTE' => $CDCLIENTE,
	        'CDCONSUMIDOR' => $CDCONSUMIDOR
        );

		$return = array('error' => false);		

		try {
        	$this->connection->executeQuery(\Util\Query::INSERT_MOVCAIXAMOB, $params);
		} catch (\Exception $e) {
			$return['error'] = true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}
		return $return;
    }
    
    public function changeTableStatus($IDSTMESAAUX, $CDFILIAL, $CDLOJA, $NRMESA, $DTBLOQMESA = null) {
		$params = array(
			'IDSTMESAAUX' => $IDSTMESAAUX, 
			'CDFILIAL' => $CDFILIAL, 
			'CDLOJA' => $CDLOJA, 
			'NRMESA' => $NRMESA,
			'DTBLOQMESA' => $DTBLOQMESA
		);
		$return = array('error' => false);		
		try {
        	$this->connection->executeQuery(\Util\Query::SQL_ALTERA_STATUS_MESA, $params);
		} catch(\Exception $e) {
			$return['error'] = true;
			$return['message']	= $e->getMessage();
		}
		return $return;
	}

    public function changePositionStatus($IDSTLUGARMESA, $CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRLUGARMESA, $DTBLOQPOS = null){
		$params = array(
			'IDSTLUGARMESA' =>$IDSTLUGARMESA,
			'CDFILIAL' =>$CDFILIAL,
			'NRVENDAREST' =>$NRVENDAREST,
			'NRCOMANDA' =>$NRCOMANDA,
			'NRLUGARMESA' =>$NRLUGARMESA,
			'DTBLOQPOS' =>$DTBLOQPOS
    	);
		$return = array('error' => false);		
		try {
        	$this->connection->executeQuery(\Util\Query::SQL_ALTERA_STATUS_POSICAO, $params);
		} catch(\Exception $e) {
			$return['error'] = true;
			$return['message']	= $e->getMessage();
		}
		return $return;
	}

    public function changePositionCpf($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRLUGARMESA, $NRINSCRCONS){
		$params = array(
			'CDFILIAL' =>$CDFILIAL,
			'NRVENDAREST' =>$NRVENDAREST,
			'NRCOMANDA' =>$NRCOMANDA,
			'NRLUGARMESA' =>$NRLUGARMESA,
			'NRINSCRCONS' =>$NRINSCRCONS
    	);
		$return = array('error' => false);		
		try {
        	$this->connection->executeQuery(\Util\Query::SQL_ALTERA_CPF_POSICAO, $params);
		} catch(\Exception $e) {
			$return['error'] = true;
			$return['message']	= $e->getMessage();
		}
		return $return;
	}

    public function getDeliveryAreas(){
    	try {
    		$params = array(
    			'SINGLE' => "'",
    			'DOUBLE' => '"'
    		);
    		if ($this->util->databaseIsOracle())$deliveryAreas = $this->connection->fetchAll(\Util\Query::ORACLE_GET_DELIVERY_AREAS, $params);
            else $deliveryAreas = $this->connection->fetchAll(\Util\Query::SQL_GET_DELIVERY_AREAS, $params);
            $return = array(
                'error' => false,
                'data'  => $deliveryAreas
            );
        } catch (\Exception $e) {
            $return = array(
                'error'   => true,
                'message' => 'Erro ao buscar areas de atendimento.Erro: '.$e->getMessage()
            );
        }
        return $return;
    }
    
    public function getProductsToRecalculate($CDFILIAL, $NRMESA, $NRLUGARMESA, $CDLOJA){
    	$params = array(		        
            'CDFILIAL' => $CDFILIAL,
            'NRMESA' => $NRMESA,
            'NRLUGARMESA' => $NRLUGARMESA,
            'CDLOJA' => $CDLOJA
        );
    	
        if ($this->util->databaseIsOracle()) $itens = $this->connection->fetchAll(\Util\Query::ORACLE_GET_PRODUCTS_RECALCULATE, $params);
        else $itens = $this->connection->fetchAll(\Util\Query::SQL_GET_PRODUCTS_RECALCULATE, $params);
        
        if(count($itens) == 0){
            $return = array(
                'error' => true,
                'message' => 'Não foram encontrado itens para esta venda'
            );
        }else{
            $return = array(
                'error' => false,
                'data' => $itens
            );
        }
        return $return;
    }

    public function getProductsToRecalculateEx($CDFILIAL, $NRCOMANDA, $CDLOJA){
    	$params = array(		        
            'CDFILIAL' => $CDFILIAL,
            'NRCOMANDA' => $NRCOMANDA,
            'CDLOJA' => $CDLOJA
        );
    	
        if ($this->util->databaseIsOracle()) $itens = $this->connection->fetchAll(\Util\Query::ORACLE_GET_PRODUCTS_RECALCULATE_EX, $params);
        else $itens = $this->connection->fetchAll(\Util\Query::SQL_GET_PRODUCTS_RECALCULATE_EX, $params);
        
        if(count($itens) == 0){
            $return = array(
                'error' => true,
                'message' => 'Não foram encontrado itens para este pedido.'
            );
        }else{
            $return = array(
                'error' => false,
                'data' => $itens
            );
        }
        return $return;
    }
    
    public function getObservations($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRPRODCOMVEN){
        $params = array(
            'CDFILIAL' => $CDFILIAL, 
            'NRVENDAREST' => $NRVENDAREST, 
            'NRCOMANDA' => $NRCOMANDA,
            'NRPRODCOMVEN' => $NRPRODCOMVEN    
        );
        
        try {
            if ($this->util->databaseIsOracle())$observations = $this->connection->fetchAll(\Util\Query::GET_LASTSALE_PRODUCT_OBSERVATIONS, $params);
            else $observations = $this->connection->fetchAll(\Util\Query::GET_POSITION_PRODUCT_OBSERVATIONS, $params);
            $return = array(
                'error' => false,
                'data' => $observations
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar observacoes. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }
    
    public function recalculateItensPrice($CDFILIAL, $CDCLIENTE, $CDPRODUTO, $CDLOJA, $CDCONSUMIDOR){
        return $this->precoAPI->buscaPreco($CDFILIAL, $CDCLIENTE, $CDPRODUTO, $CDLOJA, $CDCONSUMIDOR);
    }

    public function getCombosToRecalculate($CDFILIAL, $NRMESA, $NRLUGARMESA, $CDLOJA){
    	$params = array(
    		'CDFILIAL' => $CDFILIAL,
    		'NRMESA' => $NRMESA,
    		'NRLUGARMESA' => $NRLUGARMESA,
    		'CDLOJA' => $CDLOJA
    		
        );
        try {
        	if ($this->util->databaseIsOracle()) $combos = $this->connection->fetchAll(\Util\Query::ORACLE_GET_COMBOS_RECALCULATE, $params);
        	else $combos = $this->connection->fetchAll(\Util\Query::SQL_GET_COMBOS_RECALCULATE, $params);
            $return = array(
                'error' => false,
                'data' => $combos
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar combos relacionados à venda. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }

    public function getCombosToRecalculateEx($CDFILIAL, $NRCOMANDA, $CDLOJA){
    	$params = array(
    		'CDFILIAL' => $CDFILIAL,
    		'NRCOMANDA' => $NRCOMANDA,
    		'CDLOJA' => $CDLOJA
    		
        );
        try {
        	if ($this->util->databaseIsOracle()) $combos = $this->connection->fetchAll(\Util\Query::ORACLE_GET_COMBOS_RECALCULATE_EX, $params);
        	else $combos = $this->connection->fetchAll(\Util\Query::SQL_GET_COMBOS_RECALCULATE_EX, $params);
            $return = array(
                'error' => false,
                'data' => $combos
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar combos relacionados ao pedido. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }
    
    public function getComboProductsToRecalculate($CDPRODPROMOCAO,$CDFILIAL,$NRVENDAREST,$NRCOMANDA, $NRSEQPRODCOM){
    	$params = array(
    		'CDPRODPROMOCAO' => $CDPRODPROMOCAO,
    		'CDFILIAL' => $CDFILIAL,
    		'NRVENDAREST' => $NRVENDAREST,
    		'NRCOMANDA' => $NRCOMANDA,
    		'NRSEQPRODCOM' => $NRSEQPRODCOM
        );
        try {
            if ($this->util->databaseIsOracle()) $comboProducts = $this->connection->fetchAll(\Util\Query::ORACLE_GET_COMBO_PRODUCTS_RECALCULATE, $params);
            else $comboProducts = $this->connection->fetchAll(\Util\Query::SQL_GET_COMBO_PRODUCTS_RECALCULATE, $params);
            $return = array(
                'error' => false,
                'data' => $comboProducts
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar produtos dos combos relacionados à venda'.$e->getMessage()
            );
        }
        return $return;
    }
    
    public function printOrder($CDFILIAL, $CDLOJA, $produtos, $CDVENDEDOR, $DSCOMANDA, $NRMESA, $CDSENHAPED, $tipoVenda, $toGoString, $multiplasComandas, $ultimaComanda, $NRCOMANDA, $pedidoTaa){
		if($this->util->databaseIsOracle()){
			return true;
		}else{
			return $this->impressaoPedidoApi->imprimePedido($CDFILIAL, $CDLOJA, $produtos, $CDVENDEDOR, $DSCOMANDA, $NRMESA, $CDSENHAPED, $tipoVenda, $toGoString, $multiplasComandas, $ultimaComanda, $NRCOMANDA, $pedidoTaa);
		}
    }

    public function updateItemPrice($CDFILIAL,$NRVENDAREST,$NRCOMANDA,$NRPRODCOMVEN, $VRPRECCOMVEN, $VRDESCCOMVEN){
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'NRVENDAREST' => $NRVENDAREST,
			'NRCOMANDA' => $NRCOMANDA,
			'NRPRODCOMVEN' => $NRPRODCOMVEN,
			'VRPRECCOMVEN' => $VRPRECCOMVEN,
			'VRDESCCOMVEN' => $VRDESCCOMVEN
		);
		try {
            $success = $this->connection->executeQuery(\Util\Query::UPDATE_ITEM_PRICE, $params);
		} catch (\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro ao atualizar valor do produto. Erro: '.$e->getMessage()
			);
		}
	}

    public function getTodaysExpenditure($CDCLIENTE, $CDCONSUMIDOR){
    	$params = array(
    		'CDCLIENTE' => $CDCLIENTE,
    		'CDCONSUMIDOR' => $CDCONSUMIDOR
    	);
    	if ($this->util->databaseIsOracle()) $todaysExpenditure = $this->connection->fetchAssoc(\Util\Query::ORACLE_GET_TODAYS_EXPENDITURE,$params);
        else $todaysExpenditure = $this->connection->fetchAll(\Util\Query::SQL_GET_TODAYS_EXPENDITURE, $params);
    	return !empty($todaysExpenditure['VRMOVEXTCONS']) ? $todaysExpenditure['VRMOVEXTCONS'] : 0;
	}
	
	public function getConsumerMaxDaily($clientCode, $consumerCode){
		$params = array(
			'CDCLIENTE' => $clientCode,
			'CDCONSUMIDOR' => $consumerCode
		);
		return $this->connection->fetchAssoc(\Util\Query::GET_CONSUMER_DETAILS,$params);
	}
    /* public function makeSale(
        $NRORG, $CDFILIAL, $CDLOJA, $CDCAIXA,
        $CDVENDEDOR, $CDOPERADOR, $DTABERCAIX, $DTVENDA,
        $TOTALVENDA, $ITEMVENDA, $TIPORECE, $NMCONSVEND,
        $NRINSCRCONS, $NRSEQVENDA, $VRTROCOVEND, $EMAIL,
        $VRDESCVENDA, $CDCLIENTE, $CDCONSUMIDOR, $simulaImpressao,
        $simulateSaleValidation, $IDORIGEMVENDA, $CDSENHAPED){
        return $this->saleAPI->venda(
        $NRORG, $CDFILIAL, $CDLOJA, $CDCAIXA,
        $CDVENDEDOR, $CDOPERADOR, $DTABERCAIX, $DTVENDA,
        $TOTALVENDA, $ITEMVENDA, $TIPORECE, $NMCONSVEND,
        $NRINSCRCONS, $NRSEQVENDA, $VRTROCOVEND, $EMAIL,
        $VRDESCVENDA, $CDCLIENTE, $CDCONSUMIDOR, $simulaImpressao,
        $simulateSaleValidation, $IDORIGEMVENDA, $CDSENHAPED);
	} */
	
    public function getNRSEQVENDA($CDFILIAL, $CDCAIXA){
		$CDCONTADOR = 'PEDIDOVENDA' . $CDFILIAL . $CDCAIXA;
		if($this->util->databaseIsOracle())
			return $this->localUtil->newCode($CDCONTADOR, 10);
		else
			return $this->util->geraCodigo($this->connection, $CDCONTADOR, 1, 1, 10);
    }

    public function generateMovcaixamobSeq($CDFILIAL) {
		$movCounter = 'MOVCAIXAMOB' . $CDFILIAL;
		if($this->util->databaseIsOracle())
			return $this->localUtil->newCode($movCounter, 10);
		else
			return $this->util->geraCodigo($this->connection, $movCounter, 1, 1, 10);
    }
    
    public function generateVendarestSeq($CDFILIAL){
		$vendarestCounter = 'VENDAREST'.$CDFILIAL;
		if($this->util->databaseIsOracle())
        	return $this->localUtil->newCode($vendarestCounter, 10);
		else
			return $this->util->geraCodigo($this->connection, $vendarestCounter, 1, 1, 10);
	}

    public function generateComandavenSeq($CDFILIAL){
		$comandavenCounter = 'COMANDAVEN'.$CDFILIAL;
		if($this->util->databaseIsOracle())
        	return $this->localUtil->newCode($comandavenCounter, 10);
		else
			return $this->util->geraCodigo($this->connection, $comandavenCounter, 1, 1, 10);
		// return $this->util->geraCodigo($this->connection, $comandavenCounter, 1, 1, 10);
	}

    public function generateMovcaixadlvSeq($CDFILIAL, $NRVENDAREST){
		$movcaixadlvCounter = 'MOVCAIXADLV'.$CDFILIAL.$NRVENDAREST;
		if($this->util->databaseIsOracle())
        	return $this->localUtil->newCode($movcaixadlvCounter, 10);
		else
			return $this->util->geraCodigo($this->connection, $movcaixadlvCounter, 1, 1, 10);
	}

    public function generateItemsSeq($CDFILIAL){
		$itemsCounter = 'ITCOMANDAVEN'.$CDFILIAL;
		if($this->util->databaseIsOracle())
        	return $this->localUtil->newCode($itemsCounter, 10);
		else
			return $this->util->geraCodigo($this->connection, $itemsCounter, 1, 1, 10);
	}

    public function generateItcomandavenSeq($CDFILIAL, $NRCOMANDA){
		$itcomandavenCounter = 'ITCOMANDAVEN'.$CDFILIAL.$NRCOMANDA;
		if($this->util->databaseIsOracle())
        	return $this->localUtil->newCode($itcomandavenCounter, 6);
		else
			return $this->util->geraCodigo($this->connection, $itcomandavenCounter, 1, 1, 6);
	}

    public function generateKDSSeq($tableName){
		$kdsCounter = $tableName;
		if($this->util->databaseIsOracle())
        	return $this->localUtil->newCode($kdsCounter, 10);
		else
			return $this->util->geraCodigo($this->connection, $kdsCounter, 1, 1, 10);
	}

    public function geraSeqCdsenhaped($cdfilial, $today, $maxSeqDia){

		$CDCONTADOR = 'VENDADIA'.$cdfilial.$today;
		
        if($this->util->databaseIsOracle())
			$seqAtual = $this->localUtil->newCode($CDCONTADOR, 5);
		else
            $seqAtual = $this->util->geraCodigo($this->connection, $CDCONTADOR, 1, 1, 5);

        if (($seqAtual > $maxSeqDia['NRMAXSENHAPED'] && !empty($maxSeqDia['NRMAXSENHAPED']) || $seqAtual > 99999)){
            $params = array(
                ':CDCONTADOR' => 'VENDADIA'.$cdfilial.$today
            );
            
            $this->connection->executeQuery(\Util\Query::DELETE_CONTADOR_CDSENHAPED_DIA, $params);

            return $this->util->geraCodigo($this->connection, $CDCONTADOR, 1, 1, 5);
        }

        return $seqAtual;
    }

	public function validateTable($nrMesa){
		try{
		  $parametros = array(
			'NRMESA' => $nrMesa
		  );
		  $mesaValida = $this->connection->fetchAssoc(\Util\Query::VALIDA_MESA, $parametros);
		  if(empty($mesaValida)) throw new \Exception("Mesa não encontrada!");
		  if($mesaValida['IDSTMESAAUX'] == 'D') $this->connection->executeQuery(\Util\Query::ABRE_MESA, $parametros);
		  if($mesaValida['IDSTMESAAUX'] == 'S' || $mesaValida['IDSTMESAAUX'] == 'R') throw new \Exception("No momento a mesa se encontra bloqueada para realizar pedidos.");
		}catch(\Exception $e){
		  throw new \Exception($e->getMessage());
		}
	
	}

	public function getTableData($nrmesa, $cdfilial){
		$parametros = array(
			'NRMESA'   => $nrmesa,
			'CDFILIAL' => $cdfilial
		);
		  
		return $this->connection->fetchAssoc(\Util\Query::GET_DADOS_MESA, $parametros);
	}

	public function updateTableStatus($subsidiaryCode, $storeCode, $nrmesa){
		$parametros = array(
		  'NRMESA'   => $nrmesa,
		  'CDFILIAL' => $subsidiaryCode,
		  'CDLOJA'   => $storeCode
		);
		
		return $this->connection->executeQuery(\Util\Query::ATUALIZA_MESA, $parametros);
	}

	public function getTables($cdfilial, $cdloja, $filialvigencia, $nrconftela, $dtinivigencia){
		$params = array(
			'CDFILIAL' => $cdfilial,
            'CDLOJA' => $cdloja,
            'FILIALVIGENCIA' => $filialvigencia,
            'NRCONFTELA' => $nrconftela,
            'DTINIVIGENCIA' => $dtinivigencia
		);
		return $this->connection->fetchAll(\Util\Query::SQL_GET_TABLES, $params);
	}

	public function imprimeParcial ($produtosParcial , $cdfilial, $stVendaRests, $stComandaVens, $cdprodcouver, $modo,
		$nrvendarest, $nrcomanda, $cdloja, $nrmesa, $impPosicao, $nrpessoas, $totalProdutos, $taxaDeServico, $total, 
		$totalPorPessoa, $horas, $minutos, $segundos, $dscomanda, &$dadosImpressao, $fidelityValue, $idcomisvenda,
		$nmfanven, $idlugarmesa, $vrcomisvenda, $nrseqimprloja){
        // PRODUCT AGRUPATOR
    	$dadosComissao = $this->localUtil->getComissaoVenda($cdfilial, $nrvendarest, $nrcomanda);

		$produtosAgrupados = array();
        foreach ($produtosParcial as $produto){
            $quant = floatval($produto['QTPRODCOMVEN']);
            $key = $produto['NRLUGARMESA'].$produto['CDPRODUTO'].$produto['VRPRECCOMVEN'].$produto['VRDESCCOMVEN'].$produto['VRACRCOMVEN'];
            if ($produto['IDPESAPROD'] != 'S' && array_key_exists($key, $produtosAgrupados)){
                $produtosAgrupados[$key]['QTPRODCOMVEN'] = floatval($produtosAgrupados[$key]['QTPRODCOMVEN']) + $quant;
            }
            else {
            	if ($produto['IDPESAPROD'] == 'S') $key .= $produto['NRPRODCOMVEN'];
            	$produtosAgrupados[$key] = $produto;
            }
        }
        $produtosParcial = $produtosAgrupados;

        
		$arrPosicoes = array_filter(array_unique(array_column($produtosParcial,'NRLUGARMESA')));
		$arrNmPosicao = $this->buscaNomePorPosicao($cdfilial, $nrvendarest, $arrPosicoes);


		$valImpreLoja = $this->localUtil->getImprLoja($cdfilial, $cdloja, $nrseqimprloja);
		// Define parametros da impressora.
		$printerParams = $this->impressaoUtilAPI->buscaParametrosImpressora($valImpreLoja["IDMODEIMPRES"]);
		$strImprimir ='';
		// Monta string de impressao.
		$strImprimir .= $this->impressaoUtilAPI->centraliza($printerParams, "ACOMPANHAMENTO DO PEDIDO", $printerParams['largura']) . $printerParams['comandoEnter'];
		$strImprimir .= $this->impressaoUtilAPI->preencheLinha($printerParams, '', ' ') . $printerParams['comandoEnter'];
		if ($modo === "C") $strImprimir .= "Comanda: $dscomanda". $printerParams['comandoEnter'];

		$strImprimir .= "Mesa   : $nrmesa". $printerParams['comandoEnter'];

		$dataAtual = new \DateTime();
		$strImprimir .= "Garcom : ".$this->localUtil->removeAcentos($nmfanven). $printerParams['comandoEnter'];//nome do Garcom
		$strImprimir .= 'Emissao: '.$dataAtual->format("d/m/Y H:i:s"). $printerParams['comandoEnter'];

		$firstBlank   = true;
		$posicaoAtual = null;
		foreach ($produtosParcial as $produtoParcial){
			$posicaoMesa = $produtoParcial["NRLUGARMESA"];
			if (($idlugarmesa === "S") && ($modo === "M")){
				if ($posicaoAtual != $posicaoMesa){
					$strImprimir .= $this->impressaoUtilAPI->imprimeLinha($printerParams, $strImprimir);
					if ($firstBlank) $firstBlank = false;
					else $strImprimir .=  $printerParams['comandoEnter'];
					$strImprimir .= "Posicao ".$posicaoMesa;
					if (array_key_exists($posicaoMesa, $arrNmPosicao)){
						$nomeConsumidor = $arrNmPosicao[$posicaoMesa]['DSCONSUMIDOR'];
						$strImprimir .= ' - ' . $nomeConsumidor;
					}
					$strImprimir .= $printerParams['comandoEnter'];
					$posicaoAtual = $posicaoMesa;
				}
			}
			if ($firstBlank){
				$strImprimir .= "Posicao ".$posicaoMesa;
				if (array_key_exists($posicaoMesa, $arrNmPosicao)){
					$nomeConsumidor = $arrNmPosicao[$posicaoMesa]['DSCONSUMIDOR'];
					$strImprimir .=  ' - ' . $nomeConsumidor;
				}
				$strImprimir .= $printerParams['comandoEnter'];
				$firstBlank = false;
			}

			$strImprimir .= $produtoParcial["NMPRODUTO"]. $printerParams['comandoEnter'];
			$vrProd = round($produtoParcial["VRPRECCOMVEN"] - $produtoParcial["VRDESCCOMVEN"], 2);
			$totProd = $this->localUtil->formataPreco($produtoParcial["QTPRODCOMVEN"] * $vrProd);
			$vrProd = $this->localUtil->formataPreco($vrProd);
			$texto = $this->localUtil->toRight(str_replace(".",",",number_format($produtoParcial["QTPRODCOMVEN"], 3)) . " X " . $vrProd, $totProd, $printerParams['largura']);
			$largura = $printerParams['largura'] - strlen($texto);
			$strImprimir .= $texto. $printerParams['comandoEnter'];
		}
		$vrcomisvenda1 = floatval($vrcomisvenda);
		if ($impPosicao == '' && $nrpessoas > 1 && $idlugarmesa == "S" && $modo == "M") {
			$porcentServico = isset($dadosComissao['VRCOMISPOR']) ? $dadosComissao['VRCOMISPOR']: $vrcomisvenda1;
			$strImprimir .= $this->impressaoUtilAPI->imprimeLinha($printerParams, $strImprimir);
			$posicaoParcial = $this->localUtil->getPosicaoParcial($cdfilial, $nrvendarest, $nrcomanda);
			foreach ($posicaoParcial as $pos){
				// TOTAL PRODUTOS POSIÇÃO
				$vrTotPos = floatval($pos["PRECOTX"]) + floatval($pos["PRECONTX"]);
				$texto = "Total P" . $pos["NRLUGARMESA"] . ": " . $this->localUtil->formataPreco($vrTotPos) . " ";
				// TAXA POSIÇÃO
				if($taxaDeServico > 0) {
					$vrTxPos = (floatval($pos["PRECOTX"]) / 100) * $porcentServico;
					$texto .= "TX ". $this->localUtil->formataPreco($vrTxPos) . " ";	
				} else {
					$vrTxPos = 0;
					$texto .= "TX ". $this->localUtil->formataPreco($vrTxPos) . " ";
				}
				// TOTAL GERAL POR POSIÇÃO
				$texto .= $this->localUtil->formataPreco($vrTotPos + $vrTxPos);
				$strImprimir .= $texto . $printerParams['comandoEnter'];
			}
		}

		// lista produtos
		$strImprimir .= $this->impressaoUtilAPI->imprimeLinha($printerParams, $strImprimir);
		$texto        = "Total Produtos";
		$strImprimir .= $this->localUtil->toRight($texto, $this->localUtil->formataPreco($totalProdutos), $printerParams['largura']) . $printerParams['comandoEnter'];

		if ($fidelityValue > 0){
			$texto        = "Desconto Fidelidade";
			$strImprimir .= $this->localUtil->toRight($texto, $this->localUtil->formataPreco($fidelityValue), $printerParams['largura']) . $printerParams['comandoEnter'];
		}
		$totalPagar = round($totalProdutos - $fidelityValue, 2);
		$texto = "Subtotal";
		$strImprimir .= $this->localUtil->toRight($texto, $this->localUtil->formataPreco($totalPagar), $printerParams['largura']) . $printerParams['comandoEnter'];

		if ($taxaDeServico > 0) {
			$texto        = "Gorjeta Sugerida";
			$strImprimir .= $this->localUtil->toRight($texto, $this->localUtil->formataPreco($taxaDeServico), $printerParams['largura']) . $printerParams['comandoEnter'];
		}

		// coloca total de couvert
		$params = array($cdfilial, $nrvendarest, $nrcomanda, $cdprodcouver);
		$produtoCouvertParcial = $this->localUtil->getProdutosParcialCouver($cdfilial, $nrvendarest, $nrcomanda, $cdprodcouver);
		if ($produtoCouvertParcial["PRECO"] > 0) {
			$texto = "Couvert ";
			$strImprimir .= $texto . $this->localUtil->formataPreco($produtoCouvertParcial["PRECO"]). $printerParams['comandoEnter'];
		}
		$temp = "Total Geral";
		$strImprimir .= $this->localUtil->toRight($temp, $this->localUtil->formataPreco($total), $printerParams['largura']) . $printerParams['comandoEnter'];

		if ($idcomisvenda === 'S') {
	        $vrcomisvenda2 = null;
	        $vrcomisvenda3 = null;
	        
			if ($vrcomisvenda1 === 0.0 && $vrcomisvenda2 === 0.0 && $vrcomisvenda3 === 0.0){
				$vrcomisvenda1 = $vrcomisvenda2 = $vrcomisvenda3 = null;
			} else if ($vrcomisvenda1 === 0.0 && $vrcomisvenda2 === 0.0){
	        	$vrcomisvenda2 = null;
	        } else if ($vrcomisvenda1 === 0.0 && $vrcomisvenda3 === 0.0){
	        	$vrcomisvenda3 = null;
	        } else if ($vrcomisvenda2 === 0.0 && $vrcomisvenda3 === 0.0){
	        	$vrcomisvenda3 = null;
	        }

			if (($vrcomisvenda1 !== null && $vrcomisvenda2 !== null)  || ($vrcomisvenda1 !== null && $vrcomisvenda3 !== null) || ($vrcomisvenda2 !== null && $vrcomisvenda3 !== null)){
	            $strImprimir .= $this->impressaoUtilAPI->imprimeLinha($printerParams, $strImprimir);
	            $texto = "Gorjeta Sugerida";
	            
	            if ($vrcomisvenda1 !== null) {
		            $vrTaxa = ($totalPagar / 100) * $vrcomisvenda1;
		            $strImprimir .= $this->localUtil->toRight($texto . ' ' . $this->localUtil->formataPreco($vrcomisvenda1) . '%', $this->localUtil->formataPreco($vrTaxa + $totalPagar), $printerParams['largura']) . $printerParams['comandoEnter'];
		        }
	            if ($vrcomisvenda2 !== null) {
	                $vrTaxa = ($totalPagar / 100) * $vrcomisvenda2;
	                $strImprimir .= $this->localUtil->toRight($texto . ' ' . $this->localUtil->formataPreco($vrcomisvenda2) . '%', $this->localUtil->formataPreco($vrTaxa + $totalPagar), $printerParams['largura']) . $printerParams['comandoEnter'];
	            }
	            if ($vrcomisvenda3 !== null) {
	                $vrTaxa = ($totalPagar / 100) * $vrcomisvenda3;
	                $strImprimir .= $this->localUtil->toRight($texto . ' ' . $this->localUtil->formataPreco($vrcomisvenda3) . '%', $this->localUtil->formataPreco($vrTaxa + $totalPagar), $printerParams['largura']) . $printerParams['comandoEnter'];
	            }
	        } 
	    }

		if ($nrpessoas > 1) {
			$strImprimir .= $this->impressaoUtilAPI->imprimeLinha($printerParams, $strImprimir);
			if ($impPosicao == ''){
				$strImprimir .= "$nrpessoas PESSOA(S) - " . $this->localUtil->formataPreco($totalPorPessoa) . " POR PESSOA".$printerParams['comandoEnter'];
			}
		} else {
			$strImprimir .= $this->impressaoUtilAPI->imprimeLinha($printerParams, $strImprimir);
		}

		$strImprimir .= "Tempo de permanencia - $horas:$minutos:$segundos". $printerParams['comandoEnter'];
		$strImprimir .= $this->impressaoUtilAPI->imprimeLinha($printerParams, $strImprimir);
        $strImprimir .= $this->impressaoUtilAPI->centraliza($printerParams, 'Teknisa Software - www.teknisa.com') . $printerParams['comandoEnter'];
		$largura      = $printerParams['largura'];
		$strImprimir .= $printerParams['comandoEnter'];

		if (!$printerParams['impressaoFront']){
			$options['bold'] = true;
            $options['letterType'] = '2';
			$yesterday = date("d/m/Y", time() - 60 * 60 * 24);
			$this->localUtil->newCodeWt('VENDADIA' . $cdfilial . $yesterday, 20);
			$pedido = $this->localUtil->getNewCode('VENDADIA' . $cdfilial . $yesterday, 5);
			return $this->impressaoUtilAPI->imprimeNaoFiscal($strImprimir, $valImpreLoja, $pedido . 'D', $options);
		} else {
			$dadosImpressao = $strImprimir;			
		}
	}

	private function buscaNomePorPosicao($cdfilial, $nrvendarest, $posicoes){
        return array();
	}

	public function getWaiters(){
		$waiters = $this->connection->fetchAll(\Util\Query::GET_WAITERS);
		if(empty($waiters)) throw new \Exception("Não foram encontrados garçons.");
		return $waiters;
	}

	public function getOrderProducts($nrcomanda, $nrmesa){
		$params = array(
			'NRCOMANDA' => $nrcomanda,
			'NRMESA' => $nrmesa
		);
		$orderProducts = $this->connection->fetchAll(\Util\Query::GET_ORDER, $params);
		if (empty($orderProducts)) throw new \Exception("Comanda não encontrada.");
		foreach ($orderProducts as &$product) {
			$product['preco'] = $this->localUtil->removeDecimal($product['preco'], 3);
			$product['quantidade'] = $this->localUtil->removeDecimal($product['quantidade'], 3);
			$product['desconto'] = $this->localUtil->removeDecimal($product['desconto']);
			$product['acrescimo'] = $this->localUtil->removeDecimal($product['acrescimo']);
			$product['total'] = $this->localUtil->removeDecimal($product['total'], 3);
		}
		unset($product);
		return $orderProducts;
	}

	public function getVendorInfo($sellerCode){
		$params = array(
		  'CDVENDEDOR' => $sellerCode  
		);
		$vendor = $this->connection->fetchAssoc(\Util\Query::GET_VENDOR_INFO, $params);
		if(empty($vendor)) throw new \Exception("Vendedor não vinculado a um operador.");
		$return = array(
			'error' => false,
			'data'  => $vendor
		);
		return $return;
	}

	public function getComandaData($comandavenSeq, $subsidiaryCode){
		$params = array(
			'NRCOMANDA' => $comandavenSeq,
			'CDFILIAL' => $subsidiaryCode
		);

        $result = $this->connection->fetchAssoc(\Util\Query::GET_COMANDAVEN_DATA, $params);
		return $result;
	}

	public function getOrder($cdfilial, $nrcomanda){
		$params = array(
			'CDFILIAL' => $cdfilial,
			'NRCOMANDA' => $nrcomanda
		);
		if($this->util->databaseIsOracle()) return $this->connection->fetchAssoc(\Util\Query::GET_ORDER_DATA_ORACLE, $params);
		else return $this->connection->fetchAssoc(\Util\Query::GET_ORDER_DATA_SQL, $params);
	}

	public function getOrderPayments($cdfilial, $nrvendarest){
		$params = array(
			'CDFILIAL' => $cdfilial,
			'NRVENDAREST' => $nrvendarest
		);
		return $this->connection->fetchAll(\Util\Query::GET_ORDER_PAYMENTS, $params);
	}

	public function closeOrder($order, $cdfilial, $nrcomanda, $nrseqmovext = null) {
        try {
            $this->util->saveLog("Iniciando fechaento do pedido {$nrcomanda} - Filial {$cdfilial}", 'Service\Close', 'EXEC-');
            if(empty($this->getOrder($cdfilial, $nrcomanda)))
                throw new \Exception("O pedido ($cdfilial-$nrcomanda) não foi encontrado para fechamento.");

            if(!empty($this->checkDuplicitySale($order['CDFILIAL'], $order['CDLOJA'], $order['CDCAIXA'], $order['NRVENDAREST'], $order['NRCOMANDA']))) {
                $return = array(
                    'error' => true,
                    'message' => "O pedido ($cdfilial-$nrcomanda) já possui venda emitida."
                );

                return $return;
            }

            $cdfilial = $order['CDFILIAL'];
            $cdcaixa = $order['CDCAIXA'];
            $nrorg = $order['NRORG'];
            $nrvendarest = $order['NRVENDAREST'];
            $cdoperador = $order['CDOPERADOR'];
            $nrinscrcons = $order['NRINSCRCONS'];
            $nrmesa = $order['NRMESA'];
            $nrpesmesaven = $order['NRPESMESAVEN'];
            $dsComanda = $order['DSCOMANDA'];
            $cdloja = $order['CDLOJA'];
            $cdvendedor = $order['CDVENDEDOR'];
            $nmconsvend = $order['NMCONSUMIDOR'];
            $cdcliente = $order['CDCLIENTE'];
            $cdconsumidor = $order['CDCONSUMIDOR'];
            $idorigemvenda = $order['IDORGCMDVENDA'];
            $cdsenhaped = $order['CDSENHAPED'];
            $vrtxsevenda = (!empty($order['VRCOMISVENDE'])) ? $this->localUtil->removeDecimal($order['VRCOMISVENDE']) : 0;
            $vracrcomanda = $this->localUtil->removeDecimal($order['VRACRCOMANDA']);
            $vrdescfid = $this->localUtil->removeDecimal($order['VRDESCFID']);
            $email = null;
            $income = array();
            $change = array();
            
            $payments = $this->getOrderPayments($cdfilial, $nrvendarest);

            // $itensValue = $this->getItemsValue($cdfilial, $nrvendarest, $nrcomanda);
            $items = $this->getOrderItems($nrvendarest, $nrcomanda, $cdfilial);
            $itensValue = $this->calculaValorDaVenda($items)['total'];
            $vrDescUds = $this->getDescUDS($nrvendarest, $nrcomanda, $cdfilial); 

            foreach($payments as $payment) {
                if($payment['IDTIPOMOVIVEDLV'] == 'E') {
                    array_push($income, $payment);
                } else if($payment['IDTIPOMOVIVEDLV'] == 'S') {
                    $change = $payment;
                } 
            }
            /*
            $caixa = $this->localUtil->getCDCAIXA($cdfilial, $nrcomanda, $nrvendarest);
            $nrorg         = (!empty($caixa['NRORG'])) ? $caixa['NRORG'] : '1'; 
            if($caixa['IDCOLETOR'] === 'C') $cdcaixa = $this->localUtil->recuperaCaixaRecebedor();
            else {
                $cdcaixa = $caixa['CDCAIXACOLETOR'];
                $checaEstadoFiscal = $this->caixaAPI->checaEstadoFiscal($cdfilial, $cdcaixa, $nrorg);
            }
            
            if(empty($cdcaixa)) throw new \Exception("Não foi encontrado caixa recebedor aberto.");
        */ 
            $dtabercaix    = $this->getAberturaCaixa($cdfilial, $cdcaixa, $cdoperador);
            $dtvenda       = new \DateTime();
            $dtabercaix    = new \DateTime($dtabercaix);
            $dadosMesa     = array(
                array(
                    'NRVENDAREST'    => $nrvendarest,
                    'NRCOMANDA'      => $nrcomanda,
                    'NRMESA'         => $nrmesa,
                    'NRPESMESAVEN'   => $nrpesmesaven,
                    'DSCOMANDA'      => $dsComanda,
                    'DTHRMESAFECH'   => $dtvenda
                )
            );
            
            $lugarmesa = $this->getLugarMesa($cdfilial, $nrcomanda, $nrvendarest);
            $arrayPosicoes = array(
                $lugarmesa
            );
                
            $imprimeCupom  = false;
            $simuSaleValid = false;
            $retornoVenda = $this->vendaAPI->vendaMesa(
                $nrorg,
                $cdfilial,
                $cdloja,
                $cdcaixa,
                $cdvendedor,
                $cdoperador,
                $dtabercaix,
                $dtvenda,
                $itensValue + $vrtxsevenda,
                // $itensValue,
                $income,
                $nmconsvend,
                $nrinscrcons,
                $cdsenhaped,
                $change,
                $email,
                0,
                $cdcliente,
                $cdconsumidor,
                $imprimeCupom,
                $simuSaleValid,
                $dadosMesa,
                $arrayPosicoes,
                $idorigemvenda,
                $vrtxsevenda,
                $vrdescfid,
                0,
                null,
                null,
                null,
                true,
                $vracrcomanda,
                $nrseqmovext,
                $vrDescUds
            );
            
            $retornoVenda['CDCAIXA'] = $cdcaixa;
            $return = $retornoVenda;
        }catch (\Exception $e) {
            $return = array(
				'error' => true,
				'message' => 'Erro na finalização de pedidos. Erro: '.$e->getMessage()
			);
        }

        return $return;
	}

	public function calculaValorDaVenda($items) {
		$produtosCobrados = self::separaItensQueSeraoCobrados($items);
		$total = 0;
        // $subsidy = 0;
		foreach ($produtosCobrados as $item) {
			$total += self::calculaTotalItem($item);
            // $subsidyCalc = self::calculateSubsidy($item['VRUNITVEND'], $item['VRUNITVENDCL'], bcdiv(str_replace(',','.',strval($item['VRDESITVEND'])), str_replace(',','.',strval($item['QTPRODVEND'])), '2'));
            // $subsidy += floatval(bcmul(str_replace(',','.',strval($item['QTPRODVEND'])), str_replace(',','.',strval($subsidyCalc)), '2'));
		}
		// $produtosCancelados = self::separaItensCancelados($items);
		// $totalCancelamento = 0;
		// foreach ($produtosCancelados as $item) {
		// 	$totalCancelamento += self::calculaTotalItem($item);
		// }
		return array(
			'total' => $total,
            // 'subsidy' => $subsidy,
            // 'totalCancelamento' => $totalCancelamento
		);
	}

    public function calculaTotalItem($produtoAtual) {
		// if (isset($produtoAtual['VRDESCCOMVEN'])){
		// 	return round(floatval(bcdiv(str_replace(',','.',strval(($produtoAtual['VRPRECCOMVEN'] + $produtoAtual['VRPRECCLCOMVEN']) * $produtoAtual['QTPRODVEND'])), '1', '2')) + $produtoAtual['VRACRITVEND'] - $produtoAtual['VRDESITVEND'], 2);
		// } else {
			return round(floatval(bcmul(str_replace(',','.',strval(($produtoAtual['VRPRECCOMVEN'] + $produtoAtual['VRPRECCLCOMVEN']))), str_replace(',','.',strval($produtoAtual['QTPRODCOMVEN'])), '2')) + $produtoAtual['VRACRCOMVEN'] - $produtoAtual['VRDESCCOMVEN'], 2);
		// }
	}

    public function separaItensQueSeraoCobrados($itens) {
		$itensImposto = array();
		foreach ($itens as $itemAtual) {
			if ($itemAtual['IDSTPRCOMVEN'] !== self::ITEM_CANCELADO) {
				// se for promoção e será cobrado os itens filhos
				// if (($itemAtual['IDTIPOCOMPPROD'] == '3') && ($itemAtual['IDIMPPRODUTO'] == '2')) {
				// 	foreach ($itemAtual['itensCombo'] as $itemComboAtual) {
				// 		array_push($itensImposto, $itemComboAtual);
				// 	}
				// } else {
					array_push($itensImposto, $itemAtual);
				// }
			}
		}
		return $itensImposto;
	}
	
	private function getAberturaCaixa($cdfilial, $cdcaixa, $cdoperador) {
		$caixaInfo = $this->getCaixaInfo($cdfilial, $cdcaixa);
		$NRORG = (!empty($caixaInfo['NRORG'])) ? $caixaInfo['NRORG'] : '1';

        // Pega as informações do caixa informado
        $abertura = $this->caixaAPI->getEstadoCaixa($cdfilial, $cdcaixa, $NRORG);
		
        // Se caixa fechado, abre e retorna a data, senão, retorna a data de abertura.
        if(!empty($abertura) && $abertura['estado'] == "aberto") {
            $nrconftela = $this->getConfTela($cdcaixa, $cdfilial);
            
            if($this->util->databaseIsOracle()) $dataAbertura = \DateTime::createFromFormat('Y-m-d H:i:s', $abertura['DTABERCAIX']);
            else $dataAbertura = \DateTime::createFromFormat('Y-m-d H:i:s.u', $abertura['DTABERCAIX']);
            $today = new \DateTime();
            
            if($dataAbertura->format('d-m-Y') !== $today->format('d-m-Y')) {
                $tiporece = $this->getTiporeceFechamento($cdcaixa, $cdfilial, $dataAbertura->format('d/m/Y H:i:s'));
                $IDHABCAIXAVENDA = $caixaInfo['IDHABCAIXAVENDA'];
                $IDSINCCAIXADLV = $caixaInfo['IDSINCCAIXADLV'];
                $respostaFechamento = $this->caixaAPI->fechaCaixa('S', $dataAbertura, $today, $cdfilial, $cdcaixa, $NRORG, $cdoperador, $nrconftela, $tiporece, false, $IDHABCAIXAVENDA, $IDSINCCAIXADLV, false, false, 'S');
				
                if(!$respostaFechamento['error']) {
                    $respostaAbertura = $this->abreCaixa($cdfilial, $cdcaixa, $cdoperador);
                    if(!$respostaAbertura['error']) 
                      return $this->caixaAPI->getEstadoCaixa($cdfilial,$cdcaixa,$NRORG)['DTABERCAIX'];
                    else 
                      throw new \Exception($respostaAbertura['message']);
                } else 
                  throw new \Exception($respostaFechamento['message']);
            } else {
                return $dataAbertura->format('d-m-Y H:i:s');
            }
        } else {
            $respostaAbertura = $this->abreCaixa($cdfilial, $cdcaixa, $cdoperador);
            if(!$respostaAbertura['error'])
              return $this->caixaAPI->getEstadoCaixa($cdfilial,$cdcaixa, $NRORG)['DTABERCAIX'];
            else 
              throw new \Exception($respostaAbertura['message']);
        }
	}
	
	private function getConfTela($cdcaixa, $cdfilial){
        $parametros = array(
            'CDCAIXA' => $cdcaixa,
            'CDFILICAIXA' => $cdfilial
        );
        
        $resultado = $this->connection->fetchAssoc(\Util\Query::GET_CONFTELA_CAIXA_EX, $parametros); 
        return $resultado['NRCONFTELA'];
	}
	
	private function getTiporeceFechamento($cdcaixa, $cdfilial, $dataAbertura){
        $parametros = array(
            'CDFILIAL' => $cdfilial,
            'CDCAIXA' => $cdcaixa,
            'DTABERCAIX' => $dataAbertura
        );
        if ($this->util->databaseIsOracle()) return $this->connection->fetchAll(\Util\Query::ORACLE_GET_RECEBIMENTOS_FECHAMENTO, $parametros);
        else return $this->connection->fetchAll(\Util\Query::SQL_GET_RECEBIMENTOS_FECHAMENTO, $parametros);
	}
	
	private function abreCaixa($cdfilial, $cdcaixa, $cdoperador, $deveImprimirSuprimento = false, $forcaImpressao = true){
        // $NRORG            = "1";
		$caixaInfo = $this->getCaixaInfo($cdfilial, $cdcaixa);
		$NRORG = (!empty($caixaInfo['NRORG'])) ? $caixaInfo['NRORG'] : '1';
        $IDMONGO          = "";
        $VRMOVIVEND       = 0;
        $IDATUTURCAIXA    = "I";
        $DTABERCAIX       = new \DateTime();
		$DTMOVTURCAIX     = new \DateTime();

        $sellerReturn = $this->getStandardSeller($cdfilial);
		if(!$sellerReturn['error']) {
            $cdoperador = $sellerReturn['data']['CDOPERADOR'];
        }
		$caixaInfo = $this->getCaixaInfo($cdfilial, $cdcaixa);

		$IDHABCAIXAVENDA = $caixaInfo['IDHABCAIXAVENDA'];
		$IDSINCCAIXADLV = $caixaInfo['IDSINCCAIXADLV'];

        // Realiza a abertura de caixa.
        return $this->caixaAPI->abreCaixa(
            $cdfilial,
            $DTABERCAIX,
            $cdcaixa,
            $NRORG,
            $cdoperador,
            $DTMOVTURCAIX, 
            $VRMOVIVEND, 
            $IDMONGO, 
            $IDATUTURCAIXA, 
			$deveImprimirSuprimento,
			$IDHABCAIXAVENDA,
			$IDSINCCAIXADLV,
			$forcaImpressao
		);
	}
	
	private function getLugarMesa($cdfilial, $nrcomanda, $nrvendarest){
        $parametros = array(
            'CDFILIAL'    => $cdfilial,
            'NRCOMANDA'   => $nrcomanda,
            'NRVENDAREST' => $nrvendarest
        );
        
        $retorno = $this->connection->fetchAssoc(\Util\Query::GET_LUGARMESA, $parametros);
        return $retorno['NRLUGARMESA'];
	}
	
	public function getCaixaInfo($cdfilial, $cdcaixa){
		$parametros = array(
            'CDFILIAL' => $cdfilial,
            'CDCAIXA'  => $cdcaixa,
        );
        
        return $this->connection->fetchAssoc(\Util\Query::GET_CAIXA_INFO, $parametros);
	}

	public function getItemsValue($cdfilial, $nrvendarest, $nrcomanda){
		$params = array(
			'CDFILIAL' => $cdfilial,
			'NRVENDAREST' => $nrvendarest,
			'NRCOMANDA' => $nrcomanda
		);

		$total = $this->connection->fetchAssoc(\Util\Query::GET_ITEMS_VALUE, $params);
		return $total['TOTAL'];
	}
	
	public function cancelOrder($cdfilial, $cdcaixa, $nrseqvenda, $cdoperador, $cdsupervisor){
		$caixa = $this->getCaixaInfo($cdfilial, $cdcaixa);
		$nrorg         = (!empty($caixa['NRORG'])) ? $caixa['NRORG'] : '1'; 
		return $this->vendaAPI->cancelaCupom($cdfilial, $cdcaixa, $nrorg, $nrseqvenda, $cdoperador, $cdsupervisor);
	}
	
	public function getOrderItems($nrvendarest, $nrcomanda, $cdfilial){
		$params = array(
			'NRVENDAREST' => $nrvendarest,
			'NRCOMANDA' => $nrcomanda,
			'CDFILIAL' => $cdfilial
		);
		return $this->connection->fetchAll(\Util\Query::GET_ORDER_ITEMS, $params);
	}

    public function getDescUDS($nrvendarest, $nrcomanda, $cdfilial){
		$params = array(
			'NRVENDAREST' => $nrvendarest,
			'NRCOMANDA' => $nrcomanda,
			'CDFILIAL' => $cdfilial
		);

        $resul = $this->connection->fetchAssoc(\Util\Query::GET_CASHBACK_UDS, $params);

		return !empty($result['VRDESCCASHUDS']) ?  $this->localUtil->removeDecimal($result['VRDESCCASHUDS']) : 0 ;
	}

	public function saveCashBack($nrcomanda, $nrvendarest, $cdfilial, $cashBack){
		$params = array(
			'NRCOMANDA' => $nrcomanda,
			'NRVENDAREST' => $nrvendarest,
			'CDFILIAL' => $cdfilial,
			'VRDESCFID' => $cashBack,
			'VRDESCCASHUDS' => $cashBack
		);
		$fidelityUtil = $this->localUtil->getFidelityUtil($cdfilial);
		if($fidelityUtil['IDUTILFIDUDS'] == "S") return $this->connection->executeQuery(\Util\Query::SAVE_CASHBACK_UDS, $params);
		else return $this->connection->executeQuery(\Util\Query::SAVE_CASHBACK, $params);
	}
	
	public function updateItemDiscount($nrcomanda, $nrvendarest, $cdfilial, $cdproduto, $nrprodcomven, $discount){
		$params = array(
			'NRCOMANDA' => $nrcomanda,
			'NRVENDAREST' => $nrvendarest,
			'CDFILIAL' => $cdfilial,
			'CDPRODUTO' => $cdproduto,
			'NRPRODCOMVEN' => $nrprodcomven,
			'VRDESCCOMVEN' => $discount
		);
		return $this->connection->executeQuery(\Util\Query::UPDATE_ITEM_DISCOUNT, $params);
	}
	
	public function getOrderCashback($cdfilial, $nrvendarest, $nrcomanda){
		$params = array(
			'NRVENDAREST' => $nrvendarest,
			'CDFILIAL' => $cdfilial,
			'NRCOMANDA' => $nrcomanda
		);
		return $this->connection->fetchAssoc(\Util\Query::GET_ORDER_CASHBACK, $params); 
	}
	
	public function updateConsumerVendarest($clientCode, $consumerCode, $nrvendarest, $cdfilial){
		$params = array(
			'CDCLIENTE' => $clientCode,
			'CDCONSUMIDOR' => $consumerCode,
			'NRVENDAREST' => $nrvendarest,
			'CDFILIAL' => $cdfilial
		);
		return $this->connection->executeQuery(\Util\Query::UPDATE_CONSUMER_VENDAREST, $params);
	}
	
	public function getQuantItens($subsidiaryCode, $vendarestSeq, $comandavenSeq){
		$params = array(
			'CDFILIAL' => $subsidiaryCode,
			'NRVENDAREST' => $vendarestSeq,
			'NRCOMANDA' => $comandavenSeq	
		);
		
		$quantItens = $this->connection->fetchAssoc(\Util\Query::GET_QUANT_ITENS, $params);
		
		return empty($quantItens) ? 1 : $quantItens['QUANT'] + 1;
	}
	
	public function getOrderDscomanda($dscomanda, $subsidiaryCode){
		$dscomanda = str_pad($dscomanda, 10, 0, STR_PAD_LEFT);
		$params = array(
			'CDFILIAL' => $subsidiaryCode,
			'DSCOMANDA' => $dscomanda
		);
		
		return $this->connection->fetchAssoc(\Util\Query::GET_ORDER_DSCOMANDA, $params);
	}

    public function getOrderDscomandaFos($cdCartMifare, $subsidiaryCode){
		$params = array(
			'CDFILIAL' => $subsidiaryCode,
			'CDCARTAOMIFARE' => $cdCartMifare
		);
		
		return $this->connection->fetchAssoc(\Util\Query::GET_ORDER_DSCOMANDAFOS, $params);
	}
	
	public function getOrderById($taaSaleId){
		$params = array(
			'NRCOMANDAEXT' => $taaSaleId
		);
		return $this->connection->fetchAssoc(\Util\Query::GET_ORDER_TAA_ID, $params);
	}

	public function getStorePrinters($cdfilial, $cdloja){
		$params = array(
			'CDFILIAL' => $cdfilial,
			'CDLOJA' => $cdloja
		);
		return $this->connection->fetchAll(\Util\Query::GET_STORE_PRINTERS, $params);
	}

	public function closePos($cdfilial, $cdcaixa, $usesOpeningLock){
		$retorno = [
			'error' => false,
			'message' => '',
			'data' => []
		];
		$sellerReturn = $this->getStandardSeller($cdfilial);
		
		if(!$sellerReturn['error']) $cdoperador = $sellerReturn['data']['CDOPERADOR'];
		else throw new \Exception($sellerReturn['message']);

		// private function getAberturaCaixa($cdfilial, $cdcaixa, $cdoperador){
		$caixaInfo = $this->getCaixaInfo($cdfilial, $cdcaixa);
		$NRORG = (!empty($caixaInfo['NRORG'])) ? $caixaInfo['NRORG'] : '1';

		// Pega as informações do caixa informado
		$abertura = $this->caixaAPI->getEstadoCaixa($cdfilial, $cdcaixa, $NRORG);
		
		// Se caixa fechado, abre e retorna a data, senão, retorna a data de abertura.
		if(!empty($abertura) && $abertura['estado'] == "aberto") {
			$nrconftela = $this->getConfTela($cdcaixa, $cdfilial);
			
			if($this->util->databaseIsOracle()) $dataAbertura = \DateTime::createFromFormat('Y-m-d H:i:s', $abertura['DTABERCAIX']);
			else $dataAbertura = \DateTime::createFromFormat('Y-m-d H:i:s.u', $abertura['DTABERCAIX']);
			$today = new \DateTime();
			
			// if($dataAbertura->format('d-m-Y') !== $today->format('d-m-Y')){
				$tiporece = $this->getTiporeceFechamento($cdcaixa, $cdfilial, $dataAbertura->format('d/m/Y H:i:s'));
				
				
				$IDHABCAIXAVENDA = $caixaInfo['IDHABCAIXAVENDA'];
				$IDSINCCAIXADLV = $caixaInfo['IDSINCCAIXADLV'];
				$respostaFechamento = $this->caixaAPI->fechaCaixa('S', $dataAbertura, $today, $cdfilial, $cdcaixa, $NRORG, $cdoperador, $nrconftela, $tiporece, true, $IDHABCAIXAVENDA, $IDSINCCAIXADLV, true, false, $usesOpeningLock);
				
				if(!$respostaFechamento['error']) {
					
					// $respostaAbertura = $this->abreCaixa($cdfilial, $cdcaixa, $cdoperador);
					// if(!$respostaAbertura['error']) return $this->caixaAPI->getEstadoCaixa($cdfilial,$cdcaixa,$NRORG)['DTABERCAIX'];
					// else throw new \Exception($respostaAbertura['message']);
				} else {
          throw new \Exception($respostaFechamento['message']);
        }
        
				$retorno['data'] = $respostaFechamento;
			// }else{
			// 	return $dataAbertura->format('d-m-Y H:i:s');
			// }
		} else {
			$retorno['error'] = true;
			$retorno['message'] = "Caixa já se encontra fechado.";
		}
		return $retorno;
		// else{
		// 	$respostaAbertura = $this->abreCaixa($cdfilial, $cdcaixa, $cdoperador);
		// 	if(!$respostaAbertura['error']) return $this->caixaAPI->getEstadoCaixa($cdfilial,$cdcaixa, $NRORG)['DTABERCAIX'];
		// 	else throw new \Exception($respostaAbertura['message']);
		// }
		// }
	}

	public function openPos($cdfilial, $cdcaixa){
		$retorno = [
			'error' => false,
			'message' => '',
			'data' => []
		];
		$sellerReturn = $this->getStandardSeller($cdfilial);
		
		if(!$sellerReturn['error']) 
      $cdoperador = $sellerReturn['data']['CDOPERADOR'];
		else 
      throw new \Exception($sellerReturn['message']);
		
    $caixaInfo = $this->getCaixaInfo($cdfilial, $cdcaixa);
		$NRORG = (!empty($caixaInfo['NRORG'])) ? $caixaInfo['NRORG'] : '1';

    // Pega as informações do caixa informado
    $abertura = $this->caixaAPI->getEstadoCaixa($cdfilial, $cdcaixa, $NRORG);
    // Se caixa fechado, abre e retorna a data, senão, retorna a data de abertura.
    if(!empty($abertura) && $abertura['estado'] != "aberto") {
      $respostaAbertura = $this->abreCaixa($cdfilial, $cdcaixa, $cdoperador, true, false);
      if(!$respostaAbertura['error'])
        $retorno = $respostaAbertura;
      else
        throw new \Exception($respostaAbertura['message']);
		} else {
      $retorno['error'] = true;
			$retorno['message'] = "Caixa já se encontra aberto.";
    }
		return $retorno;
	}

	public function getPosStatus($cdfilial, $cdcaixa){
		$sellerReturn = $this->getStandardSeller($cdfilial);
		
		if(!$sellerReturn['error']) $cdoperador = $sellerReturn['data']['CDOPERADOR'];
		else throw new \Exception($sellerReturn['message']);
		$caixaInfo = $this->getCaixaInfo($cdfilial, $cdcaixa);
		$NRORG = (!empty($caixaInfo['NRORG'])) ? $caixaInfo['NRORG'] : '1';

        // Pega as informações do caixa informado
        $abertura = $this->caixaAPI->getEstadoCaixa($cdfilial, $cdcaixa, $NRORG);
		if(empty($abertura) || (!empty($abertura['error']) && $abertura['error'])) throw new \Exception((!empty($abertura['message']) ? $abertura['message'] : "Não foi possível retornar o estado do caixa."));		
		else return $abertura;
	}

	public function getSaleRouter($cdfilial, $cdcaixa, $nrseqvenda, $nrnotafiscalce, $saleCode = null){
		$params = array(
			'CDFILIAL'       => $cdfilial,
			'CDCAIXA'        => $cdcaixa,
			'NRSEQVENDA'     => $nrseqvenda, 
			'NRNOTAFISCALCE' => $nrnotafiscalce,
            'NRCOMANDAVND'   => $saleCode
		);
		if ($this->util->databaseIsOracle()) return $this->connection->fetchAssoc(\Util\Query::ORACLE_GET_SALE_ROUTER, $params);
        else return $this->connection->fetchAssoc(\Util\Query::SQL_GET_SALE_ROUTER, $params);
	}

	public function editServiceTax($cdfilial, $nrcomanda, $vrcomisvende){
		$params = array(
			'CDFILIAL' => $cdfilial,
			'NRCOMANDA' => $nrcomanda,
			'VRCOMISVENDE' => $vrcomisvende
		);
		return $this->connection->executeQuery(\Util\Query::UPDATE_VRCOMISVENDE, $params);
	}

	public function updateChildOrder($cdfilial, $nrcomandaFilha, $dscomandaPai, $idstcomanda){
		$params = array(
			'CDFILIAL' => $cdfilial,
			'NRCOMANDA' => $nrcomandaFilha,
			'DSCOMANDAPRI' => $dscomandaPai,
			'IDSTCOMANDA' => $idstcomanda
		);
		return $this->connection->executeQuery(\Util\Query::UPDATE_CHILD_ORDER, $params);
	}

	public function addItComandaven($item){
		$params = array(
			':CDFILIAL'       => $item['CDFILIAL'],
			':NRVENDAREST'    => $item['NRVENDAREST'],
			':NRCOMANDA'      => $item['NRCOMANDA'],
			':NRPRODCOMVEN'   => $item['NRPRODCOMVEN'],
			':CDPRODUTO'      => $item['CDPRODUTO'],
			':QTPRODCOMVEN'   => $item['QTPRODCOMVEN'],
			':VRPRECCOMVEN'   => $item['VRPRECCOMVEN'],
			':TXPRODCOMVEN'   => $item['TXPRODCOMVEN'],
			':IDSTPRCOMVEN'   => $item['IDSTPRCOMVEN'],
			':VRDESCCOMVEN'   => $item['VRDESCCOMVEN'],
			':NRLUGARMESA'    => $item['NRLUGARMESA'],
			':IDPRODIMPFIS'   => $item['IDPRODIMPFIS'],
			':CDLOJA'         => $item['CDLOJA'],
			':VRACRCOMVEN'    => $item['VRACRCOMVEN'],
			':NRSEQPRODCOM'   => $item['NRSEQPRODCOM'],
			':NRSEQPRODCUP'   => $item['NRSEQPRODCUP'],
			':VRPRECCLCOMVEN' => $item['VRPRECCLCOMVEN'],
			':CDCAIXACOLETOR' => $item['CDCAIXACOLETOR'],
			':CDPRODPROMOCAO' => $item['CDPRODPROMOCAO'],
			':CDVENDEDOR'     => $item['CDVENDEDOR'],
			':NRINSCRCONS'    => $item['NRINSCRCONS'],
			':CDSENHAPED'     => $item['CDSENHAPED'],
			':NRATRAPRODCOVE' => $item['NRATRAPRODCOVE'],
			':IDORIGEMVENDA'  => $item['IDORIGEMVENDA'],
			':DSOBSPEDDIGCMD' => $item['DSOBSPEDDIGCMD'],
			':IDIMPPRGRPROMO' => $item['IDIMPPRGRPROMO'],
			':DSCOMANDAORI'   => $item['DSCOMANDAORI'],
			':NRCOMANDAORI'   => $item['NRCOMANDAORI'],
			':NRPRODCOMORI'   => $item['NRPRODCOMORI']
		);
		return $this->connection->executeQuery(\Util\Query::ADD_ITCOMANDAVEN, $params);
	}

	public function deleteJoinOrderItems($cdfilial, $nrvendarest, $nrcomanda, $dscomandaori, $nrcomandaori){
		$params = array(
			':CDFILIAL'       => $cdfilial,
			':NRVENDAREST'    => $nrvendarest,
			':NRCOMANDA'      => $nrcomanda,
			':DSCOMANDAORI'   => $dscomandaori,
			':NRCOMANDAORI'   => $nrcomandaori,
		);
		return $this->connection->executeQuery(\Util\Query::DELETE_JOIN_ORDER_ITEMS, $params);
	}

	public function getChildOrders($cdfilial, $dscomanda){
		$params = array(
			':CDFILIAL' => $cdfilial,
			':DSCOMANDAPRI' => $dscomanda
		);
		return $this->connection->fetchAll(\Util\Query::GET_CHILD_ORDERS, $params);
	}

	public function cancelPayments($cdfilial, $nrvendarest){
		$params = array(
			':CDFILIAL'       => $cdfilial,
			':NRVENDAREST'    => $nrvendarest
		);
		return $this->connection->executeQuery(\Util\Query::DELETE_MOVCAIXADLV, $params);
	}

    public function editConsumer($cdfilial, $nrvendarest, $cdcliente, $cdconsumidor){
		$params = array(
			'CDFILIAL' => $cdfilial,
			'NRVENDAREST' => $nrvendarest,
			'CDCLIENTE' => $cdcliente,
            'CDCONSUMIDOR' => $cdconsumidor
		);
		return $this->connection->executeQuery(\Util\Query::UPDATE_CONSUMER_ORDER, $params);
	}

    public function getJoinedOrdersConsumers($cdfilial, $dscomanda){
		$params = array(
			':CDFILIAL' => $cdfilial,
			':DSCOMANDA' => $dscomanda
		);
		return $this->connection->fetchAll(\Util\Query::GET_JOINED_ORDERS_CONSUMERS, $params);
	}

    public function getItemsPrintCancel($cdfilial, $nrcomanda, $nrvendarest){
        $params = array(
			':CDFILIAL' => $cdfilial,
			':NRCOMANDA' => $nrcomanda,
            ':NRVENDAREST' => $nrvendarest
		);
		return $this->connection->fetchAll(\Util\Query::GET_ITEMS_PRINT_CANCEL, $params);
    }

    public function getPosPrintCancel($cdfilial, $cdcaixa){
        $params = array(
			':CDFILIAL' => $cdfilial,
            ':CDCAIXA' => $cdcaixa
		);
		return $this->connection->fetchAll(\Util\Query::GET_POS_PRINT_CANCEL, $params);
    }

    public function printCancel($order, $items, $modo, $dsComanda, $pos){
		// $session = $this->util->getSessionVars($chave);
		$dados = array();

		for ($i = 0; $i < count($items); $i++) {
			if (isset($items[$i]['NRSEQIMPRLOJA'])) {
				$dados[] = $items[$i];
			}
		}

		if (empty($dados)) return array('error' => true, 'message' => 'Nenhuma impressora vinculado aos produtos do pedido.');

		$impressora = $dados[0]['NRSEQIMPRLOJA'];
		$text = '';
		// $yesterday = date("d/m/Y", time() - 60 * 60 * 24);
		// $this->util->newCode('VENDADIA' . $session['CDFILIAL'] . $yesterday);
		// $nrPedido = $this->util->getNewCode('VENDADIA' . $session['CDFILIAL'] . $yesterday, 5);

		$oldItem = null;
		// imprime cancelamento
		$numMesa = $dados[0]['NRMESA'];
		// monta o cabeçalho do cancelamento
		//********************************************************************************
		$printerParams = $this->impressaoUtilAPI->buscaParametrosImpressora($dados[0]['IDMODEIMPRES']);
		$text .= $printerParams['comandoEnter'];
		$text .= $this->impressaoUtilAPI->imprimeLinha($printerParams);
		$text .= $this->impressaoUtilAPI->centraliza($printerParams, 'CANCELAMENTO DE PEDIDO', $printerParams['largura']) . $printerParams['comandoEnter'];
		$data = new \DateTime('NOW');
		$text .= $this->impressaoUtilAPI->centraliza($printerParams, 'DATA: ' . $data->format('d/m/Y H:i:s'), $printerParams['largura']) . $printerParams['comandoEnter'];
		$text .= $this->impressaoUtilAPI->imprimeLinha($printerParams);
		if ($modo == 'C')
			$text .= 'Comanda : ' . $dsComanda . $printerParams['comandoEnter'];
		$text .= 'Mesa    : ' . $numMesa . $printerParams['comandoEnter'];
		$text .= 'Garcom  : ' . $order['CDVENDEDOR'] . ' - ' . $order['NMFANVEN'] . $printerParams['comandoEnter'];
		$text .= $this->impressaoUtilAPI->imprimeLinha($printerParams);
		//********************************************************************************


		for ($i = 0; $i < count($dados); $i++) {

			if ($impressora != $dados[$i]['NRSEQIMPRLOJA']) {
				// quando a porta do produto trocar, deve-se imprimir e cortar o papel
				// monta rodapé
				//********************************************************************************
				if (!empty($dados[0]['MOTIVO'])) $text .= 'Motivo: ' . $dados[0]['MOTIVO'] . $printerParams['comandoEnter'];
				$text .= $this->impressaoUtilAPI->imprimeLinha($printerParams);
				//********************************************************************************
				$this->impressaoUtilAPI->imprimeNaoFiscal($text, $oldItem, $dsComanda . 'C'); // C de cancelamento
				$impressora = $dados[$i]['NRSEQIMPRLOJA'];

				$text = '';
				// monta o cabeçalho do cancelamento
				//********************************************************************************
				$printerParams = $this->impressaoUtilAPI->buscaParametrosImpressora($dados[$i]['IDMODEIMPRES']);
				$text .= $this->impressaoUtilAPI->imprimeLinha($printerParams);
				$text .= $this->impressaoUtilAPI->centraliza($printerParams, 'CANCELAMENTO DE PEDIDO', $printerParams['largura']) . $printerParams['comandoEnter'];
				$data = new \DateTime('NOW');
				$text .= $this->impressaoUtilAPI->centraliza($printerParams, 'DATA: ' . $data->format('d/m/Y H:i:s'), $printerParams['largura']) . $printerParams['comandoEnter'];
				$text .= $this->impressaoUtilAPI->imprimeLinha($printerParams);
				if ($modo == 'C')
					$text .= 'Comanda : ' . $dsComanda . $printerParams['comandoEnter'];
				$text .= 'Mesa    : ' . $numMesa . $printerParams['comandoEnter'];
				$text .= 'Garcom  : ' . $order['CDVENDEDOR'] . ' - ' . $order['NMFANVEN'] . $printerParams['comandoEnter'];
				$text .= $this->impressaoUtilAPI->imprimeLinha($printerParams);
				//********************************************************************************
			}

			$text .= ' ---> ' . $dados[$i]['QTPRODCOMVEN'] . ' ' . $dados[$i]['NMPRODUTO'] . $printerParams['comandoEnter'];
			$oldItem = $dados[$i];
		}
		// acrescenta o motivo
		if (!empty($dados[0]['MOTIVO']))
			$text .= 'Motivo: ' . $dados[0]['MOTIVO'] . $printerParams['comandoEnter'];

		// monta rodapé
		//********************************************************************************
		$text .= $this->impressaoUtilAPI->imprimeLinha($printerParams);
		$text .= $printerParams['comandoEnter'];
		$text .= $printerParams['comandoEnter'];
		$text .= $printerParams['comandoEnter'];
		$text .= $printerParams['comandoEnter'];
		$text .= $printerParams['comandoEnter'];
		//********************************************************************************

		$result = [];
		if($pos['IDIMPPEDPRCIGE'] !== 'S'){
			$result = $this->impressaoUtilAPI->imprimeNaoFiscal($text, $oldItem, $dsComanda . 'C'); // C de cancelamento
		}
		array_push($result, array('text' => $text));
		
		return $result;
	}
  public function getStatusTable($cdFilial, $cdLoja, $nrMesa){
		$params = array(
			':CDFILIAL' => $cdFilial,
			':CDLOJA'   => $cdLoja,
            ':NRMESA'   => $nrMesa
		);

		$status = $this->connection->fetchAssoc(\Util\Query::GET_TABLE_STATUS, $params);

        if(empty($status))
            throw new \Exception('Mesa não encontrada');

        return  $status;
	}

    public function getOrderByNrMesa($cdFilial, $nrMesa){
        $nrMesa = str_pad($nrMesa, 4, 0, STR_PAD_LEFT);
		$params = array(
			':CDFILIAL' => $cdFilial,
            ':NRMESA'   => $nrMesa
		);

		$status = $this->connection->fetchAssoc(\Util\Query::GET_ORDER_NRMESA, $params);

        if(empty($status))
            throw new \Exception('Mesa não encontrada');

        return  $status;
	}

    public function getStatusCmd($cdFilial, $cdLoja, $nrComanda){
		$params = array(
			':CDFILIAL'  => $cdFilial,
			':CDLOJA'    => $cdLoja,
            ':DSCOMANDA' => $nrComanda
		);

		$status = $this->connection->fetchAssoc(\Util\Query::CHECK_COMANDA, $params);

        if(!empty($status))
            return $status;

        return null;                  
	}

  public function openTable($cdFilial, $cdLoja, $nrMesa, $personalQtd, $cdCliente, $vendarestSeq, $comandavenSeq, $orderType, $dscomanda, $toGo){

        $standartSeller = $this->getResultParavendAndVendedor($cdFilial);
        if(substr($orderType, 0,3) === 'MES' && $nrMesa == $standartSeller['data']['NRMESA'])
            throw new \Exception("A mesa ($nrMesa) está parametrizada como padrão, e não pode ser utilizada na modalidade mesa.");
        
        $paramsVendaRest = array(
            'CDFILIAL'     => $cdFilial,
            'NRVENDAREST'  => $vendarestSeq,
            'CDLOJA'       => $cdLoja,
            'NRMESA'       => !empty($nrMesa) ? $nrMesa : $standartSeller['data']['NRMESA'],
            'CDVENDEDOR'   => $standartSeller['data']['CDVENDEDOR'],
            'CDOPERADOR'   => $standartSeller['data']['CDOPERADOR'],
            'NRPESMESAVEN' => $personalQtd,
            'CDCLIENTE'    => $cdCliente,
            'CDCONSUMIDOR' => null,
            'IDPEDIDOPAGO' => 'S'
        );

		$this->insertVendaRest($paramsVendaRest);

        $paramsComandaVen = array(
            'CDFILIAL'        => $cdFilial,
            'NRVENDAREST'     => $vendarestSeq,
            'NRCOMANDA'       => $comandavenSeq,
            'CDLOJA'          => $cdLoja,
            'IDSTCOMANDA'     => '1',
            'DSCOMANDA'       => $dscomanda,
            'SGSEXOCON'       => 'M',
            'TXMOTIVCANCE'    => " ",
            'VRACRCOMANDA'    => 0,
            'CDPAIS'          => null,
            'SGESTADO'        => null,
            'CDMUNICIPIO'     => null,
            'NRCEPCONSCOMAND' => null,
            'CDBAIRRO'        => null,
            'DSBAIRRO'        => null,
            'DSENDECONSCOMAN' => null,
            'DSCOMPLENDCOCOM' => null,
            'DSREFENDCONSCOM' => null,
            'IDORGCMDVENDA'   => $orderType,
            'IDRETBALLOJA'    => $toGo,
            'NRCOMANDAEXT'    => null,
            'IDSINCAGENDA'    => 'S',
            'DTHRAGENDADA'    => null,
            'IDIMPCUPEXP'     => '0',
            'DSCONSUMIDOR'    => null,
            'VRDESCFID'       => 0,
            'DSOBSCOMANDA'    => null,
            'IDFECHAMENTOAPI' => 'N'
        );
        
        $insertComandaven = $this->insertComandaVenEx($paramsComandaVen);

        if(isset($insertComandaven['error']) && $insertComandaven['error'])
          throw new \Exception($insertComandaven['message']);

        $params = array(
            'NRMESA' => $nrMesa
        );
        
        if(substr($orderType, 0,3) === 'MES')
            $this->connection->executeQuery(\Util\Query::ABRE_MESA, $params);

        $return = array(
          'saleCode'       => $paramsComandaVen['NRCOMANDA'],
          'tableNumber'    => $paramsVendaRest['NRMESA'],
          'people'         => $paramsVendaRest['NRPESMESAVEN'],
          'status'         => $paramsComandaVen['IDSTCOMANDA'],
          'saleDescription'=> $paramsComandaVen['DSCOMANDA'],
          'origin'         => $paramsComandaVen['IDORGCMDVENDA'],
        );

        return  $return;    
	}

  public function checkTable($cdFilial, $cdLoja, $nrMesa, $orderType){
      $params = array(
			':CDFILIAL' => $cdFilial,
			':CDLOJA'   => $cdLoja,
            ':NRMESA'   => $nrMesa,
            ':IDORGCMDVENDA' => $orderType.'%'
		);

		$status = $this->connection->fetchAssoc(\Util\Query::CHECK_TABLE, $params);

        if(!empty($status))
            throw new \Exception('Mesa já esta aberta em outra modalidade');

        return true;

  }

  public function getLatestSales($today, $timeLimit){
		$params = array(
			':P_INICIAL' => $timeLimit,
			':P_FINAL' => $today
		);
        $orders = $this->connection->fetchAll(\Util\Query::GET_LATEST_SALES, $params);
        $salesReturn = [];
        foreach($orders as $order){
            $emitDate = new \DateTime($order['DTVENDA']);
            $salesArray = array(
                    "subsidiaryCode" => $order['CDFILIAL'],
                    "posCode"        => $order['CDCAIXA'], 
                    "nfceCode"       => $order['NRNOTAFISCALCE'],
                    "saleSeq"        => $order['NRSEQVENDA'], 
                    "value"          => floatval($order['VRTOTVENDA']),  
                    "emitDate"       => $emitDate->format('d/m/Y H:i:s'), 
                    "saleCode"       => $order['NRCOMANDAVND'], 
                    "pagerId"        => $order['CDSENHAPED'],
                    "payment"        =>  []
            );

            $orderDetails = array(
                    ':CDFILIAL'   => $order['CDFILIAL'],
                    ':CDCAIXA'    => $order['CDCAIXA'],
                    ':NRSEQVENDA' => $order['NRSEQVENDA']   
            );
            $payments = $this->connection->fetchAll(\Util\Query::GET_PATMENTS_SALES, $orderDetails);
            foreach($payments as $payment){
                $paymentArray = array(
                        "quantParcel"   => 1, 
                        "paymentCode"   => $payment['CDTIPORECE'], 
                        "paymentName"   => $payment['NMTIPORECE'], 
                        "value"         => floatval($payment['VRMOVIVEND']), 
                        "nsu"           => $payment['CDNSUHOSTTEF']

                );
                array_push($salesArray['payment'], $paymentArray);
            }
            
            array_push($salesReturn, $salesArray);
        }      
		return $salesReturn;
	}

    public function updateOrderCpf($cdfilial, $nrcomanda, $nrvendarest, $orderCPF, $consumerName){
        $params = array(
			'CDFILIAL'     => $cdfilial,
			'NRVENDAREST'  => $nrvendarest,
			'NRCOMANDA'    => $nrcomanda,
            'NRINSCRCONS'  => $orderCPF,
            'NMCONSVEND'   => $consumerName
		);
		return $this->connection->executeQuery(\Util\Query::UPDATE_CPF_ORDER, $params);
    }
    
    public function getCashierByCode($cdOperador){
        $params = array(
			':CDOPERADOR' => $cdOperador
		);
		return $this->connection->fetchAssoc(\Util\Query::GET_CASHIER_BY_CODE, $params);
    }

    public function getCashierBySellerCode($cdVendedor){
        $params = array(
			':CDVENDEDOR' => $cdVendedor
		);
		return $this->connection->fetchAssoc(\Util\Query::GET_CASHIER_BY_SELLER_CODE, $params);
    }

    public function getCashierByEmail($dsEmailOper){
        $params = array(
			':DSEMAILOPER' => $dsEmailOper
		);
		return $this->connection->fetchAssoc(\Util\Query::GET_CASHIER_BY_EMAIL, $params);
    }
    public function encryptDelphi($passwd){
        return  $this->util->encryptDelphi($passwd);
    
    }
    public function updateSaleEmail($subsidiaryCode, $posCode, $saleSeq, $nfceCode, $email){
        $params = array(
            ':CDFILIAL'       => $subsidiaryCode,
            ':CDCAIXA'        => $posCode,
            ':NRSEQVENDA'     => $saleSeq,
            ':NRNOTAFISCALCE' => $nfceCode,
            ':DSEMAILENVCUPOM'=> $email,
            ':IDSITENVCUPOM'  => 'A'
        );

        return $this->connection->executeQuery(\Util\Query::UPDATE_SALE_EMAIL, $params);
    }

    public function saleWasExported($subsidiaryCode, $posCode, $saleSeq, $nfceCode){

        try{

            $params = array(
                ':CDFILIAL'       => $subsidiaryCode,
                ':CDCAIXA'        => $posCode,
                ':NRSEQVENDA'     => $saleSeq,
                ':NRNOTAFISCALCE' => $nfceCode,
            );

            $status =  $this->connection->fetchAssoc(\Util\Query::GET_STATUS_EXPORT_SALE, $params);

            if(isset($status['IDIMPVENDA']) && $status['IDIMPVENDA'] == 'S')
                $updateStatus = $this->updateSaleExportStatus($subsidiaryCode, $posCode, $saleSeq, $nfceCode);
                $status['IDIMPVENDA'] = 'A';
            

            $return = array(
                'error' => false,
                'data'  => $status
            );

        }catch(\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro ao alterar status de exportação da venda. Erro: '.$e->getMessage()
			);

        }

        return $return;
    }

    public function updateSaleExportStatus($subsidiaryCode, $posCode, $saleSeq, $nfceCode){

        $params = array(
            ':CDFILIAL'       => $subsidiaryCode,
            ':CDCAIXA'        => $posCode,
            ':NRSEQVENDA'     => $saleSeq,
            ':NRNOTAFISCALCE' => $nfceCode,
            ':IDIMPVENDA'     => 'A'
        );
        return $this->connection->executeQuery(\Util\Query::UPDATE_SALE_EXPORT_STATUS, $params);
    }

    public function checkSaleByEmail($subsidiaryCode, $posCode, $saleSeq, $nfceCode){

        try{

            $params = array(
                ':CDFILIAL'       => $subsidiaryCode,
                ':CDCAIXA'        => $posCode,
                ':NRSEQVENDA'     => $saleSeq,
                ':NRNOTAFISCALCE' => $nfceCode,
            );

            $status =  $this->connection->fetchAssoc(\Util\Query::GET_SALE_BY_EMAIL, $params);
            if(!empty($status['DSEMAILENVCUPOM']) && $status['IDSITENVCUPOM'] == 'A')
                throw new \Exception("Cupom já está na fila para envio ao email: {$status['DSEMAILENVCUPOM']}");
                

            $return = array(
                'error' => false,
                'data'  => $status
            );

        }catch(\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro. '.$e->getMessage()
			);

        }

        return $return;
    }

    public function getFamiliaSald($cdFilial){

        $data =  $this->connection->fetchAll(\Util\Query::GET_FAMILIASALD);

        return $data;
    }

    public function getIdSenhaCup($cdloja, $cdfilial, $cdcaixa){
        
        $params = array(
			'CDCAIXA'  => $cdcaixa,
			'CDFILIAL' => $cdfilial,
			'CDLOJA'   => $cdloja
		);

        return $this->connection->fetchAssoc(\Util\Query::GET_IDSENHACUP, $params);
    }

    public function getNrMinSenhaAl($cdloja, $cdfilial, $cdcaixa){
        
        $params = array(
			'CDCAIXA'  => $cdcaixa,
			'CDFILIAL' => $cdfilial,
			'CDLOJA'   => $cdloja
		);

        return $this->connection->fetchAssoc(\Util\Query::GET_NRMINSENHAAL, $params);
    }

    public function getNrMaxSenhaPed($cdloja, $cdfilial, $cdcaixa){
        
        $params = array(
			'CDCAIXA'  => $cdcaixa,
			'CDFILIAL' => $cdfilial,
			'CDLOJA'   => $cdloja
		);

        return $this->connection->fetchAssoc(\Util\Query::GET_NRMAXSENHAPED, $params);
    }

    public function getCdSenhaPed($cdloja, $cdfilial){
        
        $params = array(
			'CDFILIAL' => $cdfilial,
			'CDLOJA'   => $cdloja
		);

        return $this->connection->fetchAssoc(\Util\Query::GET_CDSENHAPED, $params);
    }

    public function validateOrder($order) {
        $canceledSale = false;
        if(substr($order['DSCOMANDA'], 0, 3) !== 'DLV') {
            $params = array(
                ':CDFILIAL'    => $order['CDFILIAL'],
                ':NRVENDAREST' => $order['NRVENDAREST']
            );
            
            $payment = $this->connection->fetchAll(\Util\Query::GET_PAYMENT_MOVCAIXADLV, $params);
            $params = array(
                ':CDFILIAL'    => $order['CDFILIAL'],
                ':NRVENDAREST' => $order['NRVENDAREST'],
                ':NRCOMANDA' => $order['NRCOMANDA']
            );
        
            $sale = $this->connection->fetchAll(\Util\Query::GET_SALE_DETAILS_SQL, $params);
            if(empty($payment)){
                $canceledSale = true;
            }
            
            if(!$canceledSale){
                if((!empty($sale))){
                    if (isset($sale[0]['IDSTATUSNFCE']) && $sale[0]['IDSITUVENDA'] === 'O'){
                        $params = array(
                            ':CDFILIAL'    => $order['CDFILIAL'],
                            ':CDCAIXA' => $sale[0]['CDCAIXA'],
                            ':NRSEQVENDA' => $sale[0]['NRSEQVENDA']
                        );
                        $this->connection->executeQuery(\Util\Query::UPDATE_SALE_STATUS, $params);
                        $this->localUtil->saveLog("Status da venda alterado para cancelado. ({$order['NRCOMANDA']} - {$order['DSCOMANDA']}) ", 'Error', 'CLOSE-');
                    }
                }
                $canceledSale = true;
            }
            
            if($canceledSale){
                $params = array(
                    ':CDFILIAL'    => $order['CDFILIAL'],
                    ':NRVENDAREST' => $order['NRVENDAREST']
                );
                $this->connection->executeQuery(\Util\Query::CANCEL_PAYMENTS, $params);
                $this->localUtil->saveLog("Pagamento da comanda {$order['DSCOMANDA']} cancelado", 'Error', 'CLOSE-');
            }
        }
        $return = array(
            'error' => $canceledSale ? true : false,
            'message' => 'Por favor, dirija-se a um caixa para finalizar sua comanda.'
        );

        return $return;
    }

    public function generateQrCodeCat($cdFilial, $cdCaixa, $nrseqvenda, $dtentrvenda) {

        $params = array(
                ':CDFILIAL'    => $cdFilial,
                ':CDCAIXA'     => $cdCaixa,
                ':NRSEQVENDA'  => $nrseqvenda,
                ':DTENTRVENDA' => $dtentrvenda
            );
        
        $getMealsPurchased = $this->connection->fetchAssoc(\Util\Query::GET_QTD_MEALS_PURCHASED, $params);
        $qtdMealsPurchased = isset($getMealsPurchased['VENDA']) ? $getMealsPurchased['VENDA'] : 0;

        $params = array(
            ':CDFILIAL' => $cdFilial,
            ':CDCAIXA'  => $cdCaixa
        );

        $utilQrCodeCat =  $this->connection->fetchAssoc(\Util\Query::GET_UTIL_QRCODE_CATRACA, $params);

        if((isset($utilQrCodeCat['IDGERAQRCODECAT'])) && ($utilQrCodeCat['IDGERAQRCODECAT'] === 'S' && $qtdMealsPurchased > 0)) {
            $mealId = $this->connection->fetchAssoc(\Util\Query::GET_MEAL_IDENTIFIER, $params);
            if(empty($mealId['IDREFQRCODECAT'])) {
                $return = array(
                    'error'     => true,
                    'message'   => 'Id Refeição não cadastrado ou inválido.',
                    'qrCodeCat' => ''
                );
                return $return;
            }

            $params = array(
                ':CDFILIAL'    => $cdFilial,
                ':CDCAIXA'     => $cdCaixa,
                ':DTENTRVENDA' => $dtentrvenda
            );

            $qtdSales = $this->connection->fetchAssoc(\Util\Query::GET_QTD_VENDAS_DIA, $params);
            $rangeIni = !empty($utilQrCodeCat['NRRANGEINI']) ? $utilQrCodeCat['NRRANGEINI'] : 0;
            $rangeFin = !empty($utilQrCodeCat['NRRANGEFIN']) ? $utilQrCodeCat['NRRANGEFIN'] : 0;
            $crachaId = ($this->localUtil->add($rangeIni, $qtdSales['QTVENDAS']));

            if (($rangeFin > 0) && ($crachaId > $rangeFin)) {
                $crachaId = $rangeIni;
            }
            
            $date =  str_replace("/", "-", $dtentrvenda." 16:40:00");
            $dtUnixTs = strtotime($date);

            $crachaId = str_pad($crachaId, 5, '0', STR_PAD_LEFT);
            $qtdMealsPurchased = str_pad($qtdMealsPurchased, 2, '0', STR_PAD_LEFT);
            $qrCodeCat = $crachaId . $mealId['IDREFQRCODECAT'] . $qtdMealsPurchased . $dtUnixTs;
        }

        $return = array(
            'error'   => false,
            'message' => '',
            'qrCodeCat' => isset($qrCodeCat) ? $qrCodeCat : '' 
        );
        return $return;
    }

    public function generateXmlCatraca($cdFilial, $dsComanda, $nrComanda, $event) {

        try {

            $params = array(
                ':CDFILIAL' => $cdFilial
            );
            $generateXmlCat = $this->connection->fetchAssoc(\Util\Query::GET_PARAM_CATRACA_SOLID, $params); 
            $size = !empty($generateXmlCat['NRTAMARQXML']) ? (int)$generateXmlCat['NRTAMARQXML'] : 10;

            if((isset($generateXmlCat['IDGERAARQXMLINT']) && $generateXmlCat['IDGERAARQXMLINT'] === 'S') && (isset($generateXmlCat['IDINTEGCATSOLID']) && $generateXmlCat['IDINTEGCATSOLID'] === 'S') && strpos($dsComanda, 'DLV') === false) {

                if (empty($generateXmlCat['DSDIRARQXMLINT']) || !is_dir($generateXmlCat['DSDIRARQXMLINT'])) {
                    throw new \Exception("O diretório de  destino do xml da Catraca não existe ou não foi parametrizado. Dir: {$generateXmlCat['DSDIRARQXMLINT']}");
                }

                $childOrders = $this->getChildOrders($cdFilial, $dsComanda);

                $dsComanda = substr($dsComanda, $size * -1);

                $xmlCat = new \DOMDocument('1.0');
                $xmlCat->xmlStandalone = true;
    
                $datapacket = $xmlCat->createElement('DATAPACKET');
                $datapacket->setAttribute('Version', '2.0');
    
                $metadata = $xmlCat->createElement('METADATA');
                $fields   = $xmlCat->createElement('FIELDS');
                $fieldCmd = $xmlCat->createElement('FIELD');
    
                $fieldCmd->setAttribute('attrname', 'ID_COMANDA');
                $fieldCmd->setAttribute('fieldtype', 'string');
                $fieldCmd->setAttribute('WIDTH', '20');
    
                $fieldEvent = $xmlCat->createElement('FIELD');
    
                $fieldEvent->setAttribute('attrname', 'EVENTO');
                $fieldEvent->setAttribute('fieldtype', 'string');
                $fieldEvent->setAttribute('WIDTH', '1');
    
                $fields->appendChild($fieldCmd);
                $fields->appendChild($fieldEvent);
                $metadata->appendChild($fields);
                $datapacket->appendChild($metadata);
    
                $rowdata = $xmlCat->createElement('ROWDATA');
                $row     = $xmlCat->createElement('ROW');
    
                $row->setAttribute('RowState', '4');
                $row->setAttribute('ID_COMANDA', $dsComanda);
                $row->setAttribute('EVENTO', $event);
    
                $rowdata->appendChild($row);
                $datapacket->appendChild($rowdata);
                $xmlCat->appendChild($datapacket);
                
                if (!file_exists($generateXmlCat['DSDIRARQXMLINT'] . DIRECTORY_SEPARATOR . $dsComanda.'_' . $event . '.xml')) {
                    file_put_contents($generateXmlCat['DSDIRARQXMLINT'] . DIRECTORY_SEPARATOR . $dsComanda.'_' . $event . '.xml', $xmlCat->saveXML());
                }

                if (!empty($childOrders) && $event === 'L') {
                    foreach ($childOrders as $cmd) {
                        $this->generateXmlCatraca($cdFilial, $cmd['DSCOMANDA'], $cmd['NRCOMANDA'], 'L');
                    }
                }
            }

            $return = array(
                'error'   => false,
                'message' => ''
            );

            return $return;

        } catch (\Exception $e) {
            $return = array(
                'error'   => true,
                'message' => "Erro comanda: {$dsComanda} | saleCode {$nrComanda}. " . $e->getMessage() 
            );

            return $return;
        }
    }

    public function verifyChildOrder($cdFilial, $dscomanda) {
        $params = array(
            ':CDFILIAL'     => $cdFilial,
            ':DSCOMANDAPRI' => $dscomanda
        );

        return $this->connection->fetchAssoc(\Util\Query::VERIFY_CHILD_ORDER, $params); 
    }

    public function checkDuplicitySale($cdFilial, $cdLoja, $cdCaixa, $nrVendaRest, $nrComanda) {
        $dtEntrVenda = date('d/m/Y', strtotime('-1 day'));
        $params = array(
			':CDFILIAL'     => $cdFilial,
			':CDLOJA'       => $cdLoja,
            ':CDCAIXA'      => $cdCaixa,
            ':DTENTRVENDA'  => $dtEntrVenda,
            ':NRVENDAREST'  => $nrVendaRest,
            ':NRCOMANDAVND' => $nrComanda
		);

        return $this->connection->fetchAssoc(\Util\Query::CHECK_DUPLICATE_SALE, $params);
    }

    public function updateComandaven($cdfilial, $nrvendarest, $nrcomanda, $idstcomanda) {
      $params = array(
          'IDSTCOMANDA' => $idstcomanda,
          'CDFILIAL'    => $cdfilial,
          'NRVENDAREST' => $nrvendarest,
          'NRCOMANDA'   => $nrcomanda
      );

      $this->connection->executeQuery(\Util\Query::UPDATE_COMANDAVEN, $params);
    }

    public function getSalesHistoryByPos($cdFilial, $cdCaixa, $dtEntrVenda) {
        $params = array(
            ':CDFILIAL'    => $cdFilial,
            ':CDCAIXA'     => $cdCaixa,
            ':DTENTRVENDA' => $dtEntrVenda
        );

        return $this->connection->fetchAll(\Util\Query::GET_SALES_HISTORY, $params);
    }

    public function getSaleData($cdFilial, $cdCaixa, $nrseqvenda) {
        $params = array(
            ':CDFILIAL'    => $cdFilial,
            ':CDCAIXA'     => $cdCaixa,
            ':NRSEQVENDA'  => $nrseqvenda
        );

        return $this->connection->fetchAssoc(\Util\Query::GET_SALE_DATA, $params);
    }

    public function getSalePayments($cdFilial, $cdCaixa, $nrseqvenda) {
        $params = array(
            ':CDFILIAL'    => $cdFilial,
            ':CDCAIXA'     => $cdCaixa,
            ':NRSEQVENDA'  => $nrseqvenda
        );

        $payments = $this->connection->fetchAll(\Util\Query::GET_SALE_PAYMENTS, $params);

        $return = [];
        foreach($payments as $payment) {
            $arr = [
                'paymentCode' => $payment['CDTIPORECE'],
                'paymentName' => $payment['NMTIPORECE'],
                'paymentType' => $payment['IDTIPORECE'],
                'nsu'         => $payment['CDNSUHOSTTEF']
            ];

            array_push($return, $arr);
        }
        return $return;
    }
}